@@ -1,7 +1,7 @@
 #############################################################
 #  analysis_tab.tcl  
 # -----------------------------------------------------------
-#  Copyright (C) 2003 Tresys Technology, LLC
+#  Copyright (C) 2003-2005 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -118,45 +118,30 @@ proc Apol_Analysis::delete_ResultsTab { pageID } {
         variable keepmodselect
 		
         if { [$results_notebook index $Apol_Analysis::emptyTabID] != [$results_notebook index $pageID] } {
-        	$bClose configure -state disabled
-        	update
+		$bClose configure -state disabled
+		update
 		# Get previous page index.
 		set prevPageIdx [expr [$results_notebook index $pageID] - 1]
 		set results_frame [Apol_Analysis::get_results_frame $pageID]
 		Apol_Analysis::clear_results_frame $results_frame $pageID
 		# Remove tab and its' widgets; then decrement current tab counter
 		$results_notebook delete $pageID 
-    		set currTabCount [expr $currTabCount - 1]
-     		
-    		# Raise the previous page.  If keepmodselected then raise empty tab
-		if { $prevPageIdx >= 0 } {
-		    if { $Apol_Analysis::keepmodselect } {
-			set raised [$results_notebook raise [$results_notebook page 0]]
-		    } else {
-			set raised [$results_notebook raise [$results_notebook page $prevPageIdx]]
-		    }
-		} else { 
-		    set raised [$results_notebook raise [$results_notebook page 0]]
-		}
-		if {$raised == $Apol_Analysis::emptyTabID} {
-		    Apol_Analysis::clear_options_frame $opts_frame
-		    Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame
-		    $updateButton configure -state disabled
-		    return 0
-		}
-    		# Set the search option widgets according to the now raised results tab.
-		# Only redraw the options frame if this is a new analysis type. 
-		if { $analysis_results_array($raised,mod_name) != $analysis_results_array($pageID,mod_name) } {
-			set curr_analysis_module $analysis_results_array($raised,mod_name)
-			Apol_Analysis::clear_options_frame $opts_frame
-			Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame 	 	
-		}
+		set currTabCount [expr $currTabCount - 1]
+		
 		# Remove the deleted tabs information from the content array
-    		array unset analysis_results_array "$pageID,*"
-    		set tab_deleted_flag 1
-    		Apol_Analysis::switch_results_tab $raised
-    		set tab_deleted_flag 0
-    		$bClose configure -state normal
+		array unset analysis_results_array "$pageID,*"
+		     		
+		# Raise the empty tab.  
+		set raised [$results_notebook raise [$results_notebook page 0]]
+		# The following 2 lines are disabled in this release to prevent reinitializing query criteria.
+		#Apol_Analysis::clear_options_frame $opts_frame
+		#Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame
+		$updateButton configure -state disabled
+		
+		set tab_deleted_flag 1
+		Apol_Analysis::switch_results_tab $raised
+		set tab_deleted_flag 0
+		$bClose configure -state normal
 	} 
      	update
     	return 0
@@ -331,7 +316,7 @@ proc Apol_Analysis::display_rename_tab_Dlg {pageID} {
 		return -1
 	}
     	set rename_tab_Dlg [toplevel .rename_tab_Dlg]
-   	wm protocol $rename_tab_Dlg WM_DELETE_WINDOW "destroy $rename_tab_Dlg"
+   	wm protocol $rename_tab_Dlg WM_DELETE_WINDOW " "
     	wm withdraw $rename_tab_Dlg
     	wm title $rename_tab_Dlg "Rename results tab"
     	
@@ -356,7 +341,7 @@ proc Apol_Analysis::display_rename_tab_Dlg {pageID} {
 	wm deiconify $rename_tab_Dlg
 	focus $rename_tab_entryBox
 	bind $rename_tab_Dlg <Return> "Apol_Analysis::rename_ResultsTab $pageID; destroy $rename_tab_Dlg"
-	
+	wm protocol $rename_tab_Dlg WM_DELETE_WINDOW "destroy $rename_tab_Dlg"
 	return 0
 }
 
@@ -448,12 +433,31 @@ proc Apol_Analysis::switch_results_tab { tabID } {
 	    Apol_Analysis::clear_options_frame $opts_frame
 	    Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame 	 	
 	}
-	Apol_Analysis::set_display_to_results_state $curr_analysis_module $analysis_results_array($tabID,query)
-     	set raised_tab_analysis_type $curr_analysis_module
+	set raised_tab_analysis_type $curr_analysis_module
    	$results_notebook raise $tabID
-     	return 0
+	Apol_Analysis::set_display_to_results_state $curr_analysis_module $analysis_results_array($tabID,query)
+   	Apol_Analysis::set_Focus_to_Text $tabID
 } 
 
+# ----------------------------------------------------------------------------------------
+#  Command Apol_Analysis::set_Focus_to_Text
+# ----------------------------------------------------------------------------------------
+proc Apol_Analysis::set_Focus_to_Text { tab } {
+	variable results_notebook
+	variable analysis_results_array
+	
+	if {$tab == $Apol_Analysis::emptyTabID} {
+		return	
+	}
+	if {[array exists analysis_results_array]} {
+		set curr_analysis_module $analysis_results_array($tab,mod_name)
+	   	set txt [${curr_analysis_module}::get_results_raised_tab]
+	   	focus $txt
+	}
+		
+	return 0
+}
+
 # ------------------------------------------------------------------------------
 #  Command Apol_Analysis::store_current_results_state
 # ------------------------------------------------------------------------------
@@ -482,7 +486,6 @@ proc Apol_Analysis::display_new_content { } {
 # ------------------------------------------------------------------------------
 proc Apol_Analysis::display_mod_options { mod_name opts_frame } { 
      	${mod_name}::display_mod_options $opts_frame
-     	return 0
 } 
 
 # ------------------------------------------------------------------------------
@@ -606,6 +609,15 @@ proc Apol_Analysis::get_analysis_info {mod_name} {
      	return $d_text
 } 
 
+# ------------------------------------------------------------------------------
+#  Command Apol_Analysis::get_results_raised_tab
+# ------------------------------------------------------------------------------
+proc Apol_Analysis::get_results_raised_tab {} {
+	variable results_notebook
+	
+     	return [$results_notebook raise]
+} 
+
 # ------------------------------------------------------------------------------
 #  Command Apol_Analysis::display_analysis_info
 # ------------------------------------------------------------------------------
@@ -621,13 +633,14 @@ proc Apol_Analysis::display_analysis_info {} {
 	set descriptive_text [Apol_Analysis::get_analysis_info $curr_analysis_module]
 	# Create the top-level dialog and subordinate widgets
     	toplevel $info_Dlg 
-   	wm protocol $info_Dlg WM_DELETE_WINDOW "destroy $Apol_Analysis::info_Dlg"
+   	wm protocol $info_Dlg WM_DELETE_WINDOW " "
     	wm withdraw $info_Dlg
     	wm title $info_Dlg "Analysis Description"
     	set topf  [frame $info_Dlg.topf]
     	set botf  [frame $info_Dlg.botf]
     	set sw [ScrolledWindow $topf.sw  -auto none]
-	set descrp_text [text $sw.descrp_text -height 5 -width 20 -font {helvetica 10}  -wrap word]
+	set descrp_text [text $sw.descrp_text -height 5 -width 20 -font $ApolTop::text_font \
+		-bg white -wrap word]
 	$sw setwidget $descrp_text
 	set b_ok [button $botf.b_ok -text "OK" -width 6 -command "destroy $Apol_Analysis::info_Dlg"]
 	pack $topf -side top -fill both -expand yes -padx 5 -pady 5
@@ -643,7 +656,7 @@ proc Apol_Analysis::display_analysis_info {} {
 	set height 440
 	wm geom $info_Dlg ${width}x${height}
 	wm deiconify $info_Dlg
-	
+	wm protocol $info_Dlg WM_DELETE_WINDOW "destroy $Apol_Analysis::info_Dlg"
      	return 0
 } 
 
@@ -695,10 +708,12 @@ proc Apol_Analysis::do_analysis { which } {
 		}
 	} 
 	if {$results_frame != ""} {
+		ApolTop::disable_DeleteWindow_event
 		set rt [catch {${curr_analysis_module}::do_analysis $results_frame} err] 
-		
+		ApolTop::enable_DeleteWindow_event
 		# Handle an error.
 		if {$rt != 0 && $which == "new_analysis"} { 
+			puts $err
 			ApolTop::resetBusyCursor	
 			# Re-enable buttons
 			$Apol_Analysis::newButton configure -state normal
@@ -710,6 +725,13 @@ proc Apol_Analysis::do_analysis { which } {
 	    		# Raise the previously raise tab.
 			Apol_Analysis::switch_results_tab $prev_raisedTab
 			return -1
+		} elseif {$rt != 0} {
+			# Remove the bad tab since the results frame has been cleared
+	    		set prev_Tab [$results_notebook pages \
+	    			[expr [$results_notebook index $prev_raisedTab] - 1]]
+			$results_notebook delete $prev_raisedTab
+	    		set currTabCount [expr $currTabCount - 1]
+			Apol_Analysis::switch_results_tab $prev_Tab
 		}
 	    	set raised_tab_analysis_type $curr_analysis_module
 	    	# Here store the current content of the new tab.
@@ -813,7 +835,7 @@ proc Apol_Analysis::open { } {
 proc Apol_Analysis::close { } {	
 	variable analysis_modules
     	variable analysis_listbox 
-
+	
      	foreach mod_name $analysis_modules {
 	       set mod_name [lindex $mod_name 0]
 	       if {[$analysis_listbox selection get] == $mod_name} {
@@ -864,27 +886,27 @@ proc Apol_Analysis::create { nb } {
  	
 	# Layout frames
         set frame [$nb insert end $ApolTop::analysis_tab -text "Analysis"]
-        set pw1 [PanedWindow $frame.pw1 -side left -weights extra]
-        $pw1 add -weight 1 -minsize 225
-        $pw1 add -weight 1
-	set topf  [frame [$pw1 getframe 0].topf]
-	set botf  [frame [$pw1 getframe 1].botf]
-        set pw2   [PanedWindow $topf.pw -side top -weights available]
-	$pw2 add -weight 1 -minsize 225
+        set analysis_top_pane [PanedWindow $frame.pw1 -side left -weights available]
+        $analysis_top_pane add -weight 1
+        $analysis_top_pane add
+	set analysis_top_f  [frame [$analysis_top_pane getframe 0].topf]
+	set botf  [frame [$analysis_top_pane getframe 1].botf]
+        set pw2   [PanedWindow $analysis_top_f.pw -side top -weights available]
+	$pw2 add -weight 1 
         $pw2 add -weight 3
 	# Major subframes
 	set t_left_f [TitleFrame [$pw2 getframe 0].t_left_f -text "Analysis Type"]
 	set title_opts_f [TitleFrame [$pw2 getframe 1].opts_frame -text "Analysis Options"]
-	set buttons_f [frame $topf.buttons_f]
+	set buttons_f [frame $analysis_top_f.buttons_f]
 	set b_title_f [TitleFrame $botf.b_title_f -text "Analysis Results"]
 	set b_topf [frame [$b_title_f getframe].b_topf]
 	set b_botf [frame [$b_title_f getframe].b_botf -relief sunken -bd 1]
 	
 	# Placing layout frames and major subframes
 	pack $buttons_f -side right -fill y -anchor ne -padx 2 -pady 2
-        pack $pw1 -fill both -expand yes
-        pack $pw2 -fill both -expand yes
-	pack $topf -side top -fill both -expand yes
+        pack $analysis_top_pane -fill both -expand yes
+        pack $pw2 -fill both -expand 1
+	pack $analysis_top_f -side top -fill both -expand 1
 	pack $botf -side top -fill both -expand yes 
 	pack $title_opts_f -side right -fill both -anchor ne -expand yes -padx 2
 	pack $t_left_f -side left -anchor nw -fill both -expand yes
@@ -908,7 +930,6 @@ proc Apol_Analysis::create { nb } {
 	pack $newButton $updateButton $infoButton -side top -pady 5 -anchor ne 
 		
 	set opts_frame [Apol_Analysis::create_options_frame [$title_opts_f getframe]]
-	
 	# Analysis type listbox
 	set sw_t     [ScrolledWindow [$t_left_f getframe].sw -auto none]
 	set analysis_listbox [ListBox $sw_t.analysis_listbox \
@@ -920,7 +941,7 @@ proc Apol_Analysis::create { nb } {
     	$analysis_listbox bindText <ButtonPress-1> { Apol_Analysis::mod_select }
     	
     	# Popup menu widget
-	set popupTab_Menu [menu .analysis_popup_Menu]
+	set popupTab_Menu [menu .analysis_popup_Menu -tearoff 0]
 	set tab_menu_callbacks [lappend tab_menu_callbacks {"Delete Tab" "Apol_Analysis::delete_ResultsTab"}]
 	set tab_menu_callbacks [lappend tab_menu_callbacks {"Rename Tab" "Apol_Analysis::display_rename_tab_Dlg"}]
 	 
