@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2003 Tresys Technology, LLC
+# Copyright (C) 2001-2005 Tresys Technology, LLC
 # see file 'COPYING' for use and warranty information 
 
 # TCL/TK GUI for SE Linux policy analysis
@@ -25,7 +25,7 @@ namespace eval ApolTop {
 	variable polstats 		""
 	# The version number is defined as a magical string here. This is later configured in the make environment.
 	variable gui_ver 		APOL_GUI_VERSION 
-	variable copyright_date		"2001-2004"
+	variable copyright_date		"2001-2005"
 	# install_dir is a magical string to be defined via the makefile!
 	variable apol_install_dir	APOL_INSTALL_DIR
 	variable recent_files
@@ -50,6 +50,7 @@ namespace eval ApolTop {
 	# Main window dimension defaults
         variable top_width             1000
         variable top_height            700
+	variable libsefs		0
 	
 	# Top-level dialog widgets
 	variable helpDlg
@@ -93,6 +94,7 @@ namespace eval ApolTop {
 	variable class_perms_tab	"Apol_Class_Perms"
 	variable users_tab		"Apol_Users"
 	variable initial_sids_tab	"Apol_Initial_SIDS"
+	variable file_contexts_tab	"Apol_File_Contexts"
 	variable cond_bools_tab		"Apol_Cond_Bools"
 	variable cond_rules_tab		"Apol_Cond_Rules"
 	variable policy_conf_tab	"Apol_PolicyConf"
@@ -125,6 +127,30 @@ proc ApolTop::get_install_dir {} {
 	return $ApolTop::apol_install_dir
 }
 
+proc ApolTop::get_toplevel_dialog {} {
+	return $ApolTop::mainframe
+}
+
+proc ApolTop::is_binary_policy {} {
+	if {$ApolTop::policy_type == $ApolTop::binary_policy_type} {
+		return 1
+	}
+	return 0
+}
+
+proc ApolTop::load_fc_index_file {} {
+	set rt [Apol_File_Contexts::load_fc_db]
+	if {$rt == 1} {
+		ApolTop::configure_load_index_menu_item 1
+	}
+	return 0
+}
+
+proc ApolTop::create_fc_index_file {} {
+	Apol_File_Contexts::display_create_db_dlg
+	return 0
+}
+
 ########################################################################
 # ::load_perm_map_fileDlg -- 
 #	- Called from Advanced menu
@@ -175,6 +201,36 @@ proc ApolTop::configure_edit_pmap_menu_item {enable} {
 	return 0
 }
 
+proc ApolTop::configure_load_index_menu_item {enable} {
+	variable mainframe
+	
+	if {$enable} {
+		[$mainframe getmenu fc_index_menu] entryconfigure last -label "Load Index..."
+	} else {
+		[$mainframe getmenu fc_index_menu] entryconfigure last -label "Load Index... (Not loaded)"	     
+	}
+	return 0
+}
+
+########################################################################
+# ::strip_list_of_empty_items -- takes a tcl list and checks for empty
+#	list items. If empty list items are found, it will be removed
+#	from the list and a new formatted list will be returned.
+#
+proc ApolTop::strip_list_of_empty_items {list_1} {
+	global tcl_version
+	
+	set len [llength $list_1]
+	set items ""
+	for {set i 0} {$i < $len} {incr i} {
+		if {[lindex $list_1 $i] != ""} {
+			set items [lappend items [lindex $list_1 $i]]	
+		}
+	}
+	
+	return $items
+}
+	
 proc ApolTop::disable_tkListbox { my_list_box } {
         global tk_version
 
@@ -334,6 +390,7 @@ proc ApolTop::get_tabname {tab} {
 proc ApolTop::set_Focus_to_Text { tab } {
 	variable components_nb
 	variable rules_nb
+	variable file_contexts_tab
 	
 	$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag normal
 	# The load query menu option should be enabled across all tabs. 
@@ -352,12 +409,15 @@ proc ApolTop::set_Focus_to_Text { tab } {
 		$ApolTop::rules_tab {
 			ApolTop::set_Focus_to_Text [$rules_nb raise]
 		} \
+		$ApolTop::file_contexts_tab {
+			Apol_File_Contexts::set_Focus_to_Text
+		} \
 		$ApolTop::types_tab {
 			Apol_Types::set_Focus_to_Text
 		} \
 		$ApolTop::terules_tab {
 			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag normal 
-			set raisedPage [$Apol_TE::notebook_results raise]
+			set raisedPage [Apol_TE::get_results_raised_tab]
 			if {$raisedPage != ""} {
 				Apol_TE::set_Focus_to_Text $raisedPage
 			} else {
@@ -383,6 +443,10 @@ proc ApolTop::set_Focus_to_Text { tab } {
 		$ApolTop::analysis_tab {
 			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag normal
 			$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag disabled
+			set raisedPage [Apol_Analysis::get_results_raised_tab]
+			if {$raisedPage != ""} {
+				Apol_Analysis::set_Focus_to_Text $raisedPage
+			} 
 		} \
 		$ApolTop::policy_conf_tab {
 			$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
@@ -507,6 +571,7 @@ proc ApolTop::search {} {
     	variable rules_tab 		
 	variable policy_conf_tab	
 	variable analysis_tab	
+	variable file_contexts_tab
 	
 	set raised_tab [$notebook raise]	
 	switch -- $raised_tab \
@@ -522,6 +587,9 @@ proc ApolTop::search {} {
     		$components_tab {
     			[$components_nb raise]::search $searchString $case_Insensitive $regExpr $srch_Direction
     		} \
+    		$file_contexts_tab {
+    			${file_contexts_tab}::search $searchString $case_Insensitive $regExpr $srch_Direction
+    		} \
     		default {
     			puts "Invalid raised tab!"
     		}  
@@ -608,11 +676,13 @@ proc ApolTop::_create_popup { path entryBox key } {
 
 ######################################################################
 #  Command: ApolTop::tklistbox_select_on_key_callback
-#  Arguments: Takes a tk listbox widget, its' associated list, and 
+#  Arguments: Takes a tk listbox widget,
+#	      the variable name of the associated global list, and 
 #	      the key pressed. Handles lowercase and uppercase key
 #	      values.
-#
-proc ApolTop::tklistbox_select_on_key_callback { path list_items key } {     
+# NOTE: This proc expects the associated list to be globally defined. 
+proc ApolTop::tklistbox_select_on_key_callback { path list_items_1 key } {     
+	upvar #0 $list_items_1 list_items
 	if {$path == ""} {
 		tk_messageBox \
 			-icon error \
@@ -822,7 +892,7 @@ proc ApolTop::display_searchDlg {} {
 	
 	# Create the toplevel dialog window and set its' properties.
 	toplevel $searchDlg
-	wm protocol $searchDlg WM_DELETE_WINDOW "destroy $searchDlg"
+	wm protocol $searchDlg WM_DELETE_WINDOW " "
 	wm withdraw $searchDlg
 	wm title $searchDlg "Find"
 	
@@ -876,7 +946,7 @@ proc ApolTop::display_searchDlg {} {
 	focus $searchDlg_entryBox 
 	$searchDlg_entryBox selection range 0 end
 	bind $ApolTop::searchDlg <Return> { ApolTop::search }
-	
+	wm protocol $searchDlg WM_DELETE_WINDOW "destroy $searchDlg"
 	return 0
 }	
 
@@ -916,6 +986,7 @@ proc ApolTop::call_tabs_goto_line_cmd { } {
     	variable rules_tab 		
 	variable policy_conf_tab	
 	variable analysis_tab		
+	variable file_contexts_tab
 	
 	set raised_tab [$notebook raise]	
 	switch -- $raised_tab \
@@ -931,6 +1002,9 @@ proc ApolTop::call_tabs_goto_line_cmd { } {
     		$components_tab {
     			[$components_nb raise]::goto_line $goto_line_num
     		} \
+    		$file_contexts_tab {
+    			${file_contexts_tab}::goto_line $goto_line_num
+    		} \
     		default {
     			return -code error
     		}  
@@ -951,9 +1025,9 @@ proc ApolTop::display_options_Dlg { } {
     		return 0
     	}
     	toplevel $options_Dialog
-   	wm protocol $options_Dialog WM_DELETE_WINDOW "destroy $options_Dialog"
+   	wm protocol $options_Dialog WM_DELETE_WINDOW " "
     	wm withdraw $options_Dialog
-    	wm title $options_Dialog "Tool Options"
+    	wm title $options_Dialog "Open Options"
     	
 	set open_opts_f [TitleFrame $options_Dialog.open_opts_f -text "Open policy options"]
 	set t_frame [frame [$open_opts_f getframe].t_frame]
@@ -962,17 +1036,17 @@ proc ApolTop::display_options_Dlg { } {
 	set rframe [frame $b_frame.rframe]
 	
 	set lb_textInfo [label $t_frame.lb_textInfo -justify left \
-		-text "\
-		The following are policy options used for opening \
-		a policy to control which parts of the policy are \
-		loaded.\n"]
+		-text "The following are policy options used for opening\
+		 a policy in order to control which parts of the policy are\
+		 loaded.\nPlease note: Conditional booleans and expressions\
+		 will always be loaded. Also, attributes do not apply to a binary\npolicy file.\n"]
 	set cb_all [radiobutton $lframe.cb_all -text "All" \
 		-variable ApolTop::policy_open_option -value 0 \
 		-justify left]
-	set cb_users [radiobutton $lframe.cb_users -text "Users" \
+	set cb_users [radiobutton $lframe.cb_users -text "Users (includes roles, types and attributes)" \
 		-variable ApolTop::policy_open_option -value 1 \
 		-justify left]
-	set cb_roles [radiobutton $lframe.cb_roles -text "Roles" \
+	set cb_roles [radiobutton $lframe.cb_roles -text "Roles (types and attributes included)" \
 		-variable ApolTop::policy_open_option -value 2 \
 		-justify left]
 	set cb_ta [radiobutton $lframe.cb_ta -text "Types and attributes" \
@@ -985,18 +1059,20 @@ proc ApolTop::display_options_Dlg { } {
 		-text "Classes and permissions only" \
 		-variable ApolTop::policy_open_option -value 5 \
 		-justify left]
-	set cb_rbac [radiobutton $rframe.cb_rbac -text "RBAC policy (includes roles and role rules)" \
+	set cb_rbac [radiobutton $rframe.cb_rbac -text "RBAC policy (includes roles, role rules,\n\
+		types, attributes, classes, permissions)" \
 		-variable ApolTop::policy_open_option -value 6 \
 		-justify left]
-	set cb_te [radiobutton $rframe.cb_te -text "TE policy (includes classes, permissions, types, \n\
+	set cb_te [radiobutton $rframe.cb_te -text "TE policy (includes classes, permissions, types,\n\
 		attributes and TE rules)" \
 		-variable ApolTop::policy_open_option -value 7 \
 		-justify left]
-	set cb_cond [radiobutton $rframe.cb_cond -text "Conditionals (includes conditional bools,\nconditional \
-		expressions and conditional \nTE rules)" \
+	set cb_cond [radiobutton $rframe.cb_cond -text "Conditionals (includes conditional TE rules,\
+		 types,\nattributes, classes and permissions)" \
 		-variable ApolTop::policy_open_option -value 8 \
 		-justify left]
-	set cb_sids [radiobutton $rframe.cb_sids -text "Initial SIDs" \
+	set cb_sids [radiobutton $rframe.cb_sids -text "Initial SIDs (includes types, attributes, roles, and\n\
+		users)" \
 		-variable ApolTop::policy_open_option -value 9 \
 		-justify left]
 	
@@ -1012,7 +1088,7 @@ proc ApolTop::display_options_Dlg { } {
 	# Place a toplevel at a particular position
     	#::tk::PlaceWindow $options_Dialog widget center
 	wm deiconify $options_Dialog
-	
+	wm protocol $options_Dialog WM_DELETE_WINDOW "destroy $options_Dialog"
 	return 0
 }
 
@@ -1035,7 +1111,7 @@ proc ApolTop::display_goto_line_Dlg { } {
     		return 0
     	}
     	toplevel $goto_Dialog
-   	wm protocol $goto_Dialog WM_DELETE_WINDOW "destroy $goto_Dialog"
+   	wm protocol $goto_Dialog WM_DELETE_WINDOW " "
     	wm withdraw $goto_Dialog
     	wm title $goto_Dialog "Goto"
     	
@@ -1059,7 +1135,16 @@ proc ApolTop::display_goto_line_Dlg { } {
 	wm deiconify $goto_Dialog
 	focus $gotoDlg_entryBox
 	bind $ApolTop::goto_Dialog <Return> { ApolTop::call_tabs_goto_line_cmd; destroy $ApolTop::goto_Dialog }
-	
+	wm protocol $goto_Dialog WM_DELETE_WINDOW "destroy $goto_Dialog"
+	return 0
+}
+
+proc ApolTop::check_libsefs {} {
+	set rt [catch {set ret [apol_IsLibsefs_BuiltIn]} err]
+	if {$rt != 0} {
+		return -code error $err
+	} 
+	set ApolTop::libsefs $ret
 	return 0
 }
 
@@ -1092,17 +1177,23 @@ proc ApolTop::create { } {
 	    	{} -command "ApolTop::load_query_info" }
 	    {command "&Save query..." {Disable_SaveQuery_Tag} "Save query"  \
 	    	{} -command "ApolTop::save_query_info" }
+	    {separator}
 	    {command "&Policy Summary" {Disable_Summary} "Display summary statics" {} -command ApolTop::popupPolicyStats }
 	}
 	"&Advanced" all options 0 {
 	    {cascad "&Permission Mappings" {Perm_Map_Tag} pmap_menu 0 {}}
-	    {command "&Tool Options..." {} "Tool options"  \
-	    	{} -command "ApolTop::display_options_Dlg" }
+	    #{command "Open O&ptions..." {} "Open options"  \
+	    #	{} -command "ApolTop::display_options_Dlg" }
+	    #{cascad "&File Context Indexing" {FC_Index_Tag} fc_index_menu 0 {}}
         }
 	"&Help" {} helpmenu 0 {
 	    {command "&General Help" {all option} "Show help" {} -command {ApolTop::helpDlg "Help" "apol_help.txt"}}
 	    {command "&Domain Transition Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Domain Transition Analysis Help" "dta_help.txt"}}
 	    {command "&Information Flow Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Information Flow Analysis Help" "iflow_help.txt"}}
+	    #{command "&Information Flow Assertion Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Information Flow Assertion Analysis Help" "flow_assertion_help.txt"}}
+	    {command "&File Relabel Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "File Relabel Analysis Help" "file_relabel_help.txt"}}
+	    {command "&Types Relationship Summary Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Types Relationship Summary Analysis Help" "types_relation_help.txt"}}
+	    {separator}
 	    {command "&About" {all option} "Show about box" {} -command ApolTop::aboutBox}
 	}
 	}
@@ -1114,6 +1205,10 @@ proc ApolTop::create { } {
 	[$mainframe getmenu pmap_menu] insert 0 command -label "Load Perm Map from file..." -command "ApolTop::load_perm_map_fileDlg"
 	[$mainframe getmenu pmap_menu] insert 0 separator
 	[$mainframe getmenu pmap_menu] insert 0 command -label "Load Default Perm Map" -command "ApolTop::load_default_perm_map_Dlg"
+	
+	#[$mainframe getmenu fc_index_menu] insert 0 command -label "Load Index... (Not loaded)" -command "ApolTop::load_fc_index_file"
+	#[$mainframe getmenu fc_index_menu] insert 0 command -label "Create Index" -command "ApolTop::create_fc_index_file"
+		
 	$mainframe addindicator -textvariable ApolTop::policyConf_lineno -width 14
 	$mainframe addindicator -textvariable ApolTop::polstats -width 88
 	$mainframe addindicator -textvariable ApolTop::polversion -width 19 
@@ -1121,6 +1216,7 @@ proc ApolTop::create { } {
 	# Disable menu items since a policy is not yet loaded.
 	$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag disabled
 	$ApolTop::mainframe setmenustate Perm_Map_Tag disabled
+	$ApolTop::mainframe setmenustate FC_Index_Tag normal
 	$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
 	$ApolTop::mainframe setmenustate Disable_LoadQuery_Tag disabled
 	$ApolTop::mainframe setmenustate Disable_Summary disabled
@@ -1132,6 +1228,10 @@ proc ApolTop::create { } {
 	# Create Top-level tab frames	
 	set components_frame [$notebook insert end $ApolTop::components_tab -text "Policy Components"]
 	set rules_frame [$notebook insert end $ApolTop::rules_tab -text "Policy Rules"]
+
+	if {$ApolTop::libsefs == 1} {
+		Apol_File_Contexts::create $notebook
+	}
 	Apol_Analysis::create $notebook
 	Apol_PolicyConf::create $notebook
 	
@@ -1149,8 +1249,8 @@ proc ApolTop::create { } {
 	
 	# Subtabs for the main policy rules tab
 	Apol_TE::create $rules_nb
-	Apol_RBAC::create $rules_nb
 	Apol_Cond_Rules::create $rules_nb
+	Apol_RBAC::create $rules_nb
 	
 	$components_nb compute_size
 	pack $components_nb -fill both -expand yes -padx 4 -pady 4
@@ -1513,16 +1613,16 @@ proc ApolTop::helpDlg {title file_name} {
     	destroy $helpDlg
     }
     toplevel $helpDlg
-    wm protocol $helpDlg WM_DELETE_WINDOW "destroy $helpDlg"
+    wm protocol $helpDlg WM_DELETE_WINDOW " "
     wm withdraw $helpDlg
     wm title $helpDlg "$title"
 
     set hbox [frame $helpDlg.hbox ]
     # Display results window
     set sw [ScrolledWindow $hbox.sw -auto none]
-    set resultsbox [text [$sw getframe].text -bg white -wrap none]
+    set resultsbox [text [$sw getframe].text -bg white -wrap none -font $ApolTop::text_font]
     $sw setwidget $resultsbox
-    set okButton [Button $hbox.okButton -text "OK" \
+    set okButton [Button $hbox.okButton -text "Close" \
 		      -command "destroy $helpDlg"]
     # go to the script dir to find the help file
     set script_dir  [apol_GetScriptDir "$file_name"]
@@ -1544,18 +1644,18 @@ proc ApolTop::helpDlg {title file_name} {
     	$resultsbox insert end [read $f]
     	close $f
     }
-    $resultsbox configure -state disabled
-   	 
+    ApolTop::makeTextBoxReadOnly $resultsbox
+    wm protocol $helpDlg WM_DELETE_WINDOW "destroy $helpDlg"
     return
 }
 
 proc ApolTop::makeTextBoxReadOnly {w} {
-	    $w configure -state disabled
-	    $w mark set insert 0.0
-	    $w mark set anchor insert
-	    focus $w
-	    
-	    return 0
+	$w mark set insert 0.0
+	$w mark set anchor insert
+	$w configure -state disabled
+	focus -force $w
+	
+	return 0
 }
 
 proc ApolTop::setBusyCursor {} {
@@ -1778,7 +1878,8 @@ proc ApolTop::closePolicy {} {
         Apol_Cond_Bools::close
         Apol_Cond_Rules::close
         Apol_Analysis::close 
-        Apol_PolicyConf::close      
+        Apol_PolicyConf::close    
+        
 	ApolTop::set_Focus_to_Text [$ApolTop::notebook raise]
 	set rt [catch {apol_ClosePolicy} err]
 	if {$rt != 0} {
@@ -1788,12 +1889,14 @@ proc ApolTop::closePolicy {} {
 	set policy_is_open 0
 	$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag disabled
 	# Disable Edit perm map menu item since a perm map is not yet sloaded.
-	$ApolTop::mainframe setmenustate Perm_Map_Tag disabled	
+	$ApolTop::mainframe setmenustate Perm_Map_Tag disabled
 	$ApolTop::mainframe setmenustate Disable_SaveQuery_Tag disabled
 	$ApolTop::mainframe setmenustate Disable_LoadQuery_Tag disabled
 	$ApolTop::mainframe setmenustate Disable_Summary disabled
 	ApolTop::enable_non_binary_tabs
 	ApolTop::enable_disable_conditional_widgets 1
+	ApolTop::configure_edit_pmap_menu_item 0
+	#ApolTop::configure_load_index_menu_item 0
 	
 	return 0
 }
@@ -1910,7 +2013,7 @@ proc ApolTop::set_initial_open_policy_state {} {
 		ApolTop::enable_disable_conditional_widgets 0
 	} 
 	
-	if {$ApolTop::policy_type == $ApolTop::binary_policy_type} {
+	if {[ApolTop::is_binary_policy]} {
    		ApolTop::disable_non_binary_tabs
    	}   	
    	
@@ -1919,7 +2022,6 @@ proc ApolTop::set_initial_open_policy_state {} {
 	$ApolTop::mainframe setmenustate Perm_Map_Tag normal
 	$ApolTop::mainframe setmenustate Disable_Summary normal
 	$ApolTop::mainframe setmenustate Disable_SearchMenu_Tag normal	
-	ApolTop::configure_edit_pmap_menu_item 0
 	
    	return 0
 }
@@ -2071,6 +2173,9 @@ proc ApolTop::apolExit { } {
 	if {$policy_is_open} {
 		ApolTop::closePolicy
 	}
+	if {$ApolTop::libsefs == 1} {
+		Apol_File_Contexts::close  
+	}
 	ApolTop::free_call_back_procs
 	ApolTop::writeInitFile
 	exit
@@ -2102,16 +2207,19 @@ proc ApolTop::load_fonts { } {
 	# First set all fonts in general; then change specific fonts
 	if {$general_font == ""} {
 		option add *Font "Helvetica 10"
+		set general_font "Helvetica 10"
 	} else {
 		option add *Font $general_font
 	}
 	if {$title_font == ""} {
 		option add *TitleFrame.l.font "Helvetica 10 bold italic" 
+		set title_font "Helvetica 10 bold italic"
 	} else {
 		option add *TitleFrame.l.font $title_font  
 	}
 	if {$dialog_font == ""} {
 		option add *Dialog*font "Helvetica 10" 
+		set dialog_font "Helvetica 10"
 	} else {
 		option add *Dialog*font $dialog_font
 	}
@@ -2124,6 +2232,14 @@ proc ApolTop::load_fonts { } {
 	return 0	
 }
 
+proc ApolTop::disable_DeleteWindow_event {} {
+	wm protocol . WM_DELETE_WINDOW { } 
+}
+
+proc ApolTop::enable_DeleteWindow_event {} {
+	wm protocol . WM_DELETE_WINDOW "ApolTop::apolExit"
+}
+
 proc ApolTop::main {} {
 	global tk_version
 	global tk_patchLevel
@@ -2177,7 +2293,13 @@ proc ApolTop::main {} {
 	
 	wm withdraw .
 	wm title . "SE Linux Policy Analysis"
-	wm protocol . WM_DELETE_WINDOW "ApolTop::apolExit"
+	ApolTop::enable_DeleteWindow_event
+	
+	set rt [catch {ApolTop::check_libsefs} err]
+	if {$rt != 0} {
+		tk_messageBox -icon error -type ok -title "Error" -message "$err"
+		return
+	}
 	
 	# Read apols' default settings file, gather all font information, create the gui and then load recent files into the menu.
 	ApolTop::readInitFile
