@@ -20,11 +20,13 @@
 #endif
 
 /* * Return codes for find_default_policy_file() function. */
-#define FIND_DEFAULT_SUCCESS 		0
-#define GENERAL_ERROR	 		-1
-#define BIN_POL_FILE_DOES_NOT_EXIST 	-2
-#define SRC_POL_FILE_DOES_NOT_EXIST 	-3
-#define NOT_SELINUX_AWARE		-4
+#define FIND_DEFAULT_SUCCESS 			0
+#define GENERAL_ERROR	 			-1
+#define BIN_POL_FILE_DOES_NOT_EXIST 		-2
+#define SRC_POL_FILE_DOES_NOT_EXIST 		-3
+#define BOTH_POL_FILE_DO_NOT_EXIST 		-4
+#define POLICY_INSTALL_DIR_DOES_NOT_EXIST	-5
+#define INVALID_SEARCH_OPTIONS			-6
 
 int close_policy(policy_t *policy);
 int open_policy(const char* filename, policy_t **policy);
