@@ -123,6 +123,29 @@ bool_t validate_te_query(teq_query_t *q)
 	return TRUE;
 }
 
+int policy_query_add_type(int **types, int *num_types, int type)
+{
+	bool_t add = FALSE;
+
+	/* we can't do anymore checking without the policy */
+	if (type < 0) {
+		fprintf(stderr, "type must be 0 or greater\n");
+		return -1;
+	}
+
+	if (*types) {
+		if (find_int_in_array(type, *types,
+				      *num_types) < 0) {
+			add = TRUE;
+		}
+	} else {
+		add = TRUE;
+	}
+	if (add)
+		if (add_i_to_a(type, &(*num_types), &(*types)) < 0)
+			return -1;
+	return 0;
+}
 
 /* find all rules that include a given type/attribute;
  * Doesn't set rules_b value to FALSE (assumed it has been
@@ -137,8 +160,7 @@ static int match_te_rules_idx(int  idx,
                           bool_t do_indirect,
 			  bool_t only_enabled,
                           rules_bool_t *rules_b,
-                          policy_t *policy
-                           ) 		
+                          policy_t *policy) 		
 {
 	int i;
 	int ans;
@@ -213,8 +235,7 @@ static int match_te_rules_regex(regex_t *preg,
                           bool_t do_indirect,
 			  bool_t only_enabled, 
                           rules_bool_t *rules_b,
-                          policy_t *policy
-                           ) 		
+                          policy_t *policy)	
 {
 	int i, idx_type, rt;
 	char *name;
@@ -261,8 +282,7 @@ int match_te_rules(bool_t allow_regex,
                         bool_t do_indirect,
 			bool_t only_enabled, 
                         rules_bool_t *rules_b,
-                        policy_t *policy
-                        ) 		
+                        policy_t *policy) 		
 {
 	if(allow_regex) { 
 		if(!(ta_opt == IDX_TYPE || ta_opt == IDX_ATTRIB || ta_opt == IDX_BOTH))
@@ -288,8 +308,7 @@ int match_rbac_rules(int	idx,
                      bool_t	do_indirect,
                      bool_t	tgt_is_role,
                      rbac_bool_t *b,
-                     policy_t	*policy
-                    )
+                     policy_t	*policy)
 {
 	int i;
 	int ans;
@@ -323,6 +342,70 @@ int match_rbac_rules(int	idx,
 	return 0;
 }
 
+/* the behaviour of this function matches the above, however instead of returning a boolean array
+ * with the indices matching rules the indices match roles.
+ */
+
+int match_rbac_roles(int	idx,
+                     int	type,
+                     unsigned char whichlist,
+                     bool_t	do_indirect,
+                     bool_t	tgt_is_role,
+                     rbac_bool_t *b,
+                     policy_t	*policy)
+{
+	int i;
+	int ans;
+	ta_item_t *ta;
+
+	if(b == NULL)
+		return -1;
+	
+	if((whichlist & (SRC_LIST ^ TGT_LIST)) && !((whichlist & TGT_LIST) && !tgt_is_role) ) {
+		for(i = 0; i < policy->num_role_allow; i++) {
+			if (does_role_allow_use_role(idx, whichlist, do_indirect, &(policy->role_allow[i]), &(b->a_cnt))) {
+                                if (whichlist & TGT_LIST)
+                                        ta = policy->role_allow[i].src_roles;
+				else
+					ta = policy->role_allow[i].tgt_roles;
+
+				while (ta) {
+					b->allow[ta->idx] = TRUE;
+					ta = ta->next;
+				}
+			}
+		}
+	}
+
+	if(!((whichlist & TGT_LIST) && tgt_is_role) ) {
+		for(i = 0; i < policy->num_role_trans; i++) {
+			if(whichlist & (SRC_LIST ^ DEFAULT_LIST)) {
+				if (does_role_trans_use_role(idx, whichlist, do_indirect, &(policy->role_trans[i]), &(b->t_cnt))) {
+
+					if (whichlist & SRC_LIST) {
+						ta = policy->role_trans[i].src_roles;
+						while (ta) {
+							b->trans[ta->idx] = TRUE;
+							ta = ta->next;
+						}
+					}
+                                        else
+						b->trans[policy->role_trans[i].trans_role.idx] = TRUE;
+
+				}
+			}
+			if(!(b->trans[i]) && (whichlist & TGT_LIST) && !tgt_is_role) {
+				ans = does_role_trans_use_ta(idx, type, do_indirect, &(policy->role_trans[i]), 
+						&(b->t_cnt), policy);
+				if (ans == -1)
+					return -1;
+				b->trans[i] = ans;
+			}
+		}
+	}
+	
+	return 0;
+}
 
 /* search and return type enforcement rules based on provided query critiera.  Results are return
  * as arrays of rules indicies 
@@ -681,7 +764,7 @@ err_return1:
  *	-1 on error
  *	0 on success
  */
-int search_conditional_expressions(char *bool, bool_t regex, bool_t *exprs_b, char **error_msg, policy_t *policy)
+int search_conditional_expressions(bool_t use_bool, char *bool, bool_t regex, bool_t *exprs_b, char **error_msg, policy_t *policy)
 {
 	int i, rt;
 	cond_expr_t *cur;
@@ -708,13 +791,15 @@ int search_conditional_expressions(char *bool, bool_t regex, bool_t *exprs_b, ch
 		for (cur = policy->cond_exprs[i].expr; cur != NULL; cur = cur->next) {
 			if (cur->expr_type != COND_BOOL)
 				continue;
-			if (regex) {
+			if (use_bool && regex) {
 				rt = regexec(&reg, policy->cond_bools[cur->bool].name, 0, NULL, 0);
 				if (rt == 0)
 					exprs_b[i] = TRUE;
-			} else {
+			} else if (use_bool) {
 				if (strcmp(bool, policy->cond_bools[cur->bool].name) == 0)
 					exprs_b[i] = TRUE;
+			} else {
+				exprs_b[i] = TRUE;
 			}
 		}
 	}
