@@ -20,6 +20,7 @@
 #include "cond.h"
 #include "avl-util.h"
 #include "util.h"
+#include "semantic/avhash.h"
 #include <asm/types.h>
 
 #include <stdio.h>
@@ -95,7 +96,7 @@ typedef struct name_item {
 
 
 
-/* IDs for ta_item */
+/* IDs for policy components */
 #define IDX_INVALID		0x0
 #define IDX_TYPE		0x00000001
 #define IDX_ATTRIB		0x00000002
@@ -103,8 +104,10 @@ typedef struct name_item {
 #define IDX_PERM		0x00000008
 #define IDX_COMMON_PERM		0x00000010
 #define IDX_OBJ_CLASS		0x00000020
-#define IDX_SUBTRACT		0x00000040
+#define IDX_USER		0x00000040
+#define IDX_BOOLEAN		0x00000080
 #define IDX_BOTH		0x10000000	/* reserve as special indicator for regex rule searches  */
+#define IDX_SUBTRACT		0x20000000
 
 /* type, attribute, role, perm , common perm list item */
 typedef struct ta_item {
@@ -131,19 +134,12 @@ typedef struct type_item {
 } type_item_t;
 
 /* attributes and their associated types */
-typedef struct attrib_item {
+typedef struct name_a {
 	char	*name; 
-	int	num_types;
-	int	*types;  /* dynamic array of type indicies*/
-} attrib_item_t;
+	int	num;
+	int	*a;  /* dynamic array of type indicies*/
+} name_a_t;
 
-/* NOTE: the role declaration structure looks identical to the  attrib
- * delcation structure.  So rather than rebuild functions that do the 
- * same thing, we will use attrib_item_t also for role declations. */
- 
-/* FIX: At some point for code clarity, should make the attrib/role
- * functions have more generic names...very low priority! */
-typedef attrib_item_t role_item_t;
 
 typedef struct common_perm {
 	char	*name;
@@ -160,40 +156,23 @@ typedef struct obj_class {
 } obj_class_t;
 
 
-/* user statements; linked list */
-typedef struct user_item {
-	char			*name;
-	ta_item_t		*roles;
-	struct user_item	*next;
-	void 			*data;	/* generic pointer used by libseuser; ignored in apol */
-} user_item_t;
-
-typedef struct user_list {
-	user_item_t	*head;
-	user_item_t	*tail;
-} user_list_t;
-
-
 typedef struct security_context {
-	user_item_t	*user;	/* Note, unfortunately we don't have idx's for users. 
-				 * Never free user from this structure; this is just a
-				 * pointer to the real user record */
+	int		user;	
 	int		role;
 	int		type;
-} security_context_t;
+} security_con_t;
 
 /* IDs of rules */
 #define RULE_TE_ALLOW		0 	/*AV rule */
 #define RULE_AUDITALLOW		1	/*AV rule */
 #define RULE_AUDITDENY		2	/*AV rule */
-/* removed NOTIFY and replaced with DONTAUDIT  Jul 2002 */
-#define RULE_NOTIFY		3	/*AV rule */
-#define RULE_DONTAUDIT		3	/*replaces NOTIFY */
+#define RULE_DONTAUDIT		3	/* really the same as auditdeny */
 #define RULE_NEVERALLOW		4	/*AV rule */
 #define RULE_MAX_AV		4   	/*end of AV rule types */
 #define RULE_TE_TRANS		5   	/*TT rule (type transition|change|member) */
 #define RULE_TE_MEMBER		6   	/*TT rule */
 #define RULE_TE_CHANGE		7	/*TT rule */
+#define RULE_MAX_TE		7
 #define RULE_CLONE		8	/*clone rule */
 #define RULE_ROLE_ALLOW		9	/* Role allow */
 #define RULE_ROLE_TRANS		10	/* Role transition */
@@ -210,8 +189,6 @@ typedef struct security_context {
 #define AVFLAG_SRC_STAR		0x02
 #define AVFLAG_TGT_TILDA	0x04
 #define AVFLAG_TGT_STAR		0x08
-#define AVFLAG_CLS_TILDA	0x10
-#define AVFLAG_CLS_STAR		0x20
 #define AVFLAG_PERM_TILDA	0x40
 #define AVFLAG_PERM_STAR	0x80
 
@@ -221,12 +198,13 @@ typedef struct av_item {
  	unsigned char	flags;		/* where we handle '~' and '*' */
 	bool_t		enabled;	/* whether the rule is enabled or not for conditionals */
 	int		cond_expr;	/* the conditional expression that owns this rule */
+	bool_t		cond_list;	/* if cond, TRUE mean on the true list; FALSE the false list; ignore for uncond */
  	unsigned long	lineno;		/* line # from policy.conf */
  	ta_item_t	*src_types;	/* the domain types/attribs */
  	ta_item_t	*tgt_types;	/* the object types/attribs */
 	ta_item_t	*classes;
 	ta_item_t	*perms;
-} av_item_t; 	
+} av_item_t;
 
 /* structure for a type transition (member, change) rule */
 typedef struct tt_item {
@@ -234,6 +212,7 @@ typedef struct tt_item {
 	unsigned char	flags;		/* use AV* flags above, only need SRC, TGT, & CLS */
 	bool_t		enabled;	/* whether the rule is enabled or not for conditionals */
 	int		cond_expr;	/* the conditional expression that owns this rule */
+	bool_t		cond_list;	/* if cond, TRUE mean on the true list; FALSE the false list; ignore for uncond */
  	unsigned long	lineno;		/* line # from policy.conf */
 	ta_item_t	*src_types;	/* the domain types/attribs */
  	ta_item_t	*tgt_types;	/* the object types/attribs */
@@ -256,7 +235,7 @@ typedef struct rt_item {
 	ta_item_t	*src_roles;
 	ta_item_t	*tgt_types;	
 	ta_item_t	trans_role;	/* role to transition to */
-} rt_item_t;	
+} rt_item_t;
 
 /* clone rule structure; we store the rule and then resolve its semanitcs as necessary */
 typedef struct cln_item {
@@ -273,7 +252,7 @@ typedef struct cln_item {
 typedef struct initial_sid {
 	char			*name;
 	__u32			sid;
-	security_context_t 	*scontext;
+	security_con_t 	*scontext;
 } initial_sid_t;
 
 /* type alias array
@@ -299,7 +278,8 @@ typedef struct alias_item {
 #define POL_LIST_INITIAL_SIDS	12
 #define POL_LIST_COND_BOOLS	13
 #define POL_LIST_COND_EXPRS	14
-#define POL_NUM_LISTS		15
+#define POL_LIST_USERS		15
+#define POL_NUM_LISTS		16
 
 /* These are our weak indicators of which version of policy we're using.
  * The syntax and semantics of a policy are in great flux, and many changes
@@ -343,6 +323,7 @@ typedef struct policy {
 	int	num_cond_exprs;		/* " " */
 	int	num_roles;		/* " " */
 	int	num_role_allow;		/* " " */
+	int	num_users;		/* " " */
 	int	num_role_trans;		/* " " */
 	int 	num_perms;		/* " " */
 	int	num_common_perms;
@@ -363,7 +344,7 @@ typedef struct policy {
 /* Type Enforcement Rules */
 	type_item_t 	*types;    	/* defined types and their attribs (ARRAY)*/
 	alias_item_t	*aliases;	/* separate index into type aliases (ARRAY) */
-	attrib_item_t 	*attribs;	/* defined attribs and their types (ARRAY)*/
+	name_a_t 	*attribs;	/* defined attribs and their types (ARRAY)*/
 	av_item_t	*av_access;	/* allow and neverallow rules; we expect lots of these (ARRAY)*/
 	av_item_t	*av_audit;	/* audit and notify rules (ARRAY)*/
 	tt_item_t	*te_trans;	/* type transition|member|change rules (ARRAY)*/
@@ -374,18 +355,29 @@ typedef struct policy {
 	cond_bool_t	*cond_bools;	/* conditional policy booleans (ARRAY) */
 	cond_expr_item_t *cond_exprs;	/* conditional expressions (ARRAY) */
 /* Role-based access control rules */
-	role_item_t	*roles;		/* roles (ARRAY)*/
+	name_a_t	*roles;		/* roles (ARRAY)*/
 	role_allow_t	*role_allow;	/* role allow rules (ARRAY) */
 	rt_item_t	*role_trans;	/* role transition rules (ARRAY) */
 /* User rules */
-/* TODO: We probably need to re-work user into an ARRAY like most of the other lists to allow 
- *	 for growth in the number of users and make everything consistent.  Unfortunately, we
- *	 already have a large investment in code that uses the linked list!! */
-	user_list_t	users;		/* users (LLIST) */
+	name_a_t	*users;		/* users (ARRAY) */
 /* Permissions map (which is used for information flow analysis) */
 	struct classes_perm_map *pmap;	/* see perm-map.h */
+/* Semantic hash table; this structure is created when necessary (NULL ptr means its necessary)
+ * and is used to semantically check TE (av and type) rules in policy */
+ 	avh_t		avh;		/* TE rule hash table */
 } policy_t;
 
+/*
+ * obj_perm_set_t allows the inclusion/exclusion 
+ * of individual permissions on object classes or entire object 
+ * classes. 
+ */
+typedef struct obj_perm_set {
+	int obj_class;   	/* index policy->obj_classes */
+	int num_perms;
+	int *perms;    		/* index of an object class' permission */
+} obj_perm_set_t;
+
 
 /* used for whichlist arguments */
 #define SRC_LIST	0x0001
@@ -405,9 +397,11 @@ int init_policy( policy_t **policy_ptr);
 int free_policy(policy_t **policy_ptr);
 int set_policy_version(int ver, policy_t *policy);
 const char* get_policy_version_name(int policy_version);
+int get_policy_version_num(policy_t *policy);
 
 #define is_binary_policy(policy) (policy != NULL ? (policy->policy_type & POL_TYPE_BINARY) : 0)
 #define is_valid_policy_version(version) (version >= POL_VER_UNKNOWN && version <= POL_VER_MAX)
+#define get_policy_version_id(policy) (policy != NULL ? policy->version : -1)
 
 /* DB updates/additions/changes */
 #define add_common_perm_to_class(cls_idx, cp_idx, policy) ((is_valid_obj_class_idx(cls_idx, policy) && is_valid_common_perm_idx(cp_idx, policy)) ? policy->obj_classes[cls_idx].common_perms = cp_idx: -1 )
@@ -416,7 +410,6 @@ int add_type(char *type, policy_t *policy);
 int add_alias(int type_idx, char *alias, policy_t *policy);
 int add_attrib_to_type(int type_idx, char *token, policy_t *policy);
 int insert_ta_item(ta_item_t *newitem, ta_item_t **list);
-int append_user(user_item_t *newuser, user_list_t *list);
 int add_name(char *name, name_item_t **list);
 int add_clone_rule(int src, int tgt,  unsigned long lineno, policy_t *policy);
 int add_attrib(bool_t with_type, int type_idx, policy_t *policy, char *attrib);
@@ -476,7 +469,7 @@ bool_t does_class_indirectly_use_perm(int cls_idx, int perm_idx, policy_t *polic
 int add_initial_sid(char *name, policy_t *policy);
 int add_initial_sid2(char *name, __u32 sid, policy_t *policy);
 int get_initial_sid_idx(const char *name, policy_t *policy);
-int add_initial_sid_context(int idx, security_context_t *scontext, policy_t *policy);
+int add_initial_sid_context(int idx, security_con_t *scontext, policy_t *policy);
 int get_initial_sid_name(int idx, char **name, policy_t *policy);
 int search_initial_sids_context(int **isids, int *num_isids, const char *user, const char *role, const char *type, policy_t *policy);
 
@@ -501,36 +494,42 @@ int get_type_or_attrib_idx(const char *name, int *idx_type, policy_t *policy);
 int get_type_name(int idx, char **name, policy_t *policy);
 int get_attrib_name(int idx, char **name, policy_t *policy);
 int get_type_attribs(int type, int *num_attribs, int **attribs, policy_t *policy);
+int get_type_users(int type, int *num_users, int **users, policy_t *policy);
+int get_type_roles(int type, int *num_roles, int **roles, policy_t *policy);
 int get_attrib_types(int attrib, int *num_types, int **types, policy_t *policy);
+bool_t is_attrib_in_type(const char *attrib, int type_idx, policy_t *policy);
+bool_t is_type_in_attrib(const char *type, int attrib_idx, policy_t *policy);
+
 
 /* conditional policy */
-#define is_valid_cond_bool_idx(idx, policy) (policy != NULL ? (idx >= 0 && idx < policy->num_cond_bools) : 0)
+#define is_valid_cond_bool_idx(idx, policy) (policy != NULL && (idx >= 0 && idx < policy->num_cond_bools))
+#define is_valid_cond_expr_idx(idx, policy) (policy != NULL && (idx >= 0 && idx < num_cond_exprs(policy)))
 #define num_cond_bools(policy) (policy != NULL ? policy->num_cond_bools : -1)
 #define num_cond_exprs(policy) (policy != NULL ? policy->num_cond_exprs : -1)
 #define get_cond_bool_default_state(idx, policy) (is_valid_cond_bool_idx(idx, policy) ? policy->cond_bools[idx].default_state : 0)
 int get_cond_bool_idx(const char *name, policy_t *policy);
-int set_cond_bool_val(int bool, bool_t state, policy_t *policy);
-int get_cond_bool_val(char *name, policy_t *policy);
 int get_cond_bool_name(int idx, char **name, policy_t *policy);
+int set_cond_bool_val(int bool, bool_t state, policy_t *policy);
+int get_cond_bool_val(const char *name, bool_t *val, policy_t *policy);
+int get_cond_bool_val_idx(int idx, bool_t *val, policy_t *policy);
+int get_cond_bool_default_val(const char *name, bool_t *val, policy_t *policy);
+int get_cond_bool_default_val_idx(int idx, bool_t *val, policy_t *policy);
 int update_cond_expr_items(policy_t *policy);
+void add_cond_expr_item_helper(int cond_expr, bool_t cond_list, cond_rule_list_t *list, policy_t *policy);
 int set_cond_bool_vals_to_default(policy_t *policy);
 
 /* users */
-#define get_first_user_ptr(policy) ((policy != NULL) ? policy->users.head : NULL)
-#define get_next_user_ptr(user) ((user != NULL) ? user->next : NULL)
-#define get_user_name_ptr(user) ((user != NULL) ? user->name : NULL)
-#define get_user_first_role_ptr(user) ((user != NULL) ? user->roles : NULL)
-#define get_user_next_role_ptr(role) ((role != NULL) ? role->next : NULL)
 #define num_users(policy) (policy != NULL ? policy->rule_cnt[RULE_USER] : -1)
+#define is_valid_user_idx(idx, policy) (policy != NULL && (idx >= 0 && idx < policy->num_users))
 
-user_item_t *add_user(char *user, policy_t *policy);
-int add_role_to_user(user_item_t *user, int role_idx, policy_t *policy);
-int free_user(user_item_t *ptr);
-int get_user_name(user_item_t *user, char **name);
+int add_role_to_user(int role_idx, int user_idx, policy_t *policy);
+int add_user(char *user, policy_t *policy);
+bool_t does_user_have_role(int user, int role, policy_t *policy);
+bool_t is_role_in_user(const char *role, int user_idx, policy_t *policy);
 bool_t does_user_exists(const char *name, policy_t *policy);
-int get_user_by_name(const char *name, user_item_t **user, policy_t *policy);
-bool_t does_user_have_role(user_item_t *user, int role, policy_t *policy);
-
+int get_user_idx(const char *name, policy_t *policy);
+int get_user_name2(int idx, char **name, policy_t *policy);
+int get_user_roles(int user, int *num_roles, int **roles, policy_t *policy);
 
 /* roles */
 #define is_valid_role_idx(idx, policy) (policy != NULL && (idx >= 0 && idx < policy->num_roles))
@@ -541,19 +540,24 @@ int add_role(char *role, policy_t *policy);
 int get_role_name(int idx, char **name, policy_t *policy);
 int get_role_idx(const char *name, policy_t *policy);
 bool_t does_role_use_type(int role, int type, policy_t *policy);
-bool_t is_role_in_list(int role, ta_item_t *list);
-
+bool_t is_type_in_role(const char *type, int role_idx, policy_t *policy);
 
 /* TE rules */
 /* if rule_type == 1, then access rules, otherwise audit rules */
 #define is_valid_av_rule_idx(idx, rule_type, policy) (idx >= 0 && ( (rule_type == 1) ? idx < policy->num_av_access : idx < policy->num_av_audit) )
 #define is_valid_tt_rule_idx(idx, policy) (idx >= 0 && idx < policy->num_te_trans)
+#define is_cond_rule(rule) (rule.cond_expr >= 0)
+#define is_av_rule_type(type) ((type >= RULE_TE_ALLOW) && (type <= RULE_MAX_AV))
+#define is_av_access_rule_type(type) ((type == RULE_TE_ALLOW) || (type == RULE_NEVERALLOW))
+#define is_av_audit_rule_type(type) ((type >= RULE_AUDITALLOW) && (type <= RULE_MAX_AV))
+#define is_type_rule_type(type) ((type >= RULE_TE_TRANS) && (type <= RULE_MAX_TE))
+#define is_te_rule_type(type) ((type >= RULE_TE_ALLOW) && (type <= RULE_MAX_TE))
 
 int extract_types_from_te_rule(int rule_idx, int rule_type, unsigned char whichlist, int **types, int *num_types, policy_t *policy);
 int extract_obj_classes_from_te_rule(int rule_idx, int rule_type, int **obj_classes, int *num_obj_classes, policy_t *policy);
 int extract_perms_from_te_rule(int rule_idx, int rule_type, int **perms, int *num_perms, policy_t *policy);
 int does_av_rule_idx_use_type(int rule_idx, unsigned char rule_type, int type_idx, int ta_type, 
-		unsigned char whichlist, bool_t do_indirect, policy_t *policy);
+			      unsigned char whichlist, bool_t do_indirect, policy_t *policy);
 int does_av_rule_use_type(int idx, int type, unsigned char whichlist, bool_t do_indirect, 
 	av_item_t *rule, int *cnt, policy_t *policy);
 int does_tt_rule_use_type(int idx, int type, unsigned char whichlist, bool_t do_indirect, tt_item_t *rule, int *cnt, policy_t *policy);
@@ -566,6 +570,16 @@ bool_t does_role_trans_use_role(int idx, unsigned char whichlist, bool_t do_indi
 bool_t does_role_allow_use_role(int src, unsigned char whichlist,  bool_t do_indirect, role_allow_t *rule, int *cnt);
 int does_role_trans_use_ta(int idx, int type, bool_t do_indirect, rt_item_t *rule, int *cnt, policy_t *policy);
 
+/* Generic function prototypes for adding object class options and end types to a policy query. */
+int apol_obj_perm_set_init(obj_perm_set_t *it);
+void apol_free_obj_perm_set_data(obj_perm_set_t *it);
+int apol_add_class_to_obj_perm_set_list(obj_perm_set_t **obj_options, 
+				 int *num_obj_options, int obj_class);
+int apol_add_perm_to_obj_perm_set_list(obj_perm_set_t **obj_options, 
+				      int *num_obj_options, int obj_class, 
+				      int perm);
+
+
 
 /* misc */
 
@@ -573,10 +587,13 @@ bool_t does_clone_rule_use_type(int idx, int type, unsigned char whichlist, cln_
 	int *cnt, policy_t *policy);
 int get_rule_lineno(int rule_idx, int rule_type, policy_t *policy);
 int get_ta_item_name(ta_item_t *ta, char **name, policy_t *policy);
+int free_ta_list(ta_item_t *list);
+
+int get_complement(ta_item_t *, int, int *, int **);
 
 /**************/
 /* these are INTERNAL functions only; allow direct access to type/attrib name string
- * stored within the policy db.  They are expoerted only for use by other internal
+ * stored within the policy db.  They are exported only for use by other internal
  * policy db modules.
  *
  * THE CALLER SHOULD NOT FREE OR OTHERWISE MODIFY THE RETURNED STRING!!!!
