@@ -98,9 +98,10 @@ void trim_trailing_whitespace(char **str)
 /* create a new initialize list */
 llist_t *ll_new(void)
 {
-	llist_t *ll;
+	llist_t *ll = NULL;
+	
 	ll = (llist_t *)malloc(sizeof(llist_t));
-	if(ll == NULL) {
+	if (ll == NULL) {
 		fprintf(stderr, "out of memory");
 		return NULL;
 	}
@@ -256,25 +257,29 @@ int init_rules_bool(bool_t include_audit, rules_bool_t *rules_b, policy_t *polic
 	return 0;
 }
 
-int init_rbac_bool(rbac_bool_t *b, policy_t *policy) 
+/* 20050106 Added roles to argument list since I may want to create 
+ * this with either the number of rules or the number of roles 
+ */
+
+int init_rbac_bool(rbac_bool_t *b, policy_t *policy, bool_t roles) 
 {
 	if(b == NULL)
 		return -1;
 		
-	b->allow = (bool_t *)malloc(policy->num_role_allow * sizeof(bool_t));
+	b->allow = (bool_t *)malloc(roles? policy->num_roles : policy->num_role_allow * sizeof(bool_t));
 	if(b->allow == NULL) {
 		fprintf(stderr, "out of memory\n");
 		return -1;
 	}
-	memset(b->allow, 0, policy->num_role_allow * sizeof(bool_t));
+	memset(b->allow, 0, roles? policy->num_roles : policy->num_role_allow * sizeof(bool_t));
 	b->a_cnt = 0;
 	
-	b->trans = (bool_t *)malloc(policy->num_role_trans * sizeof(bool_t));
+	b->trans = (bool_t *)malloc(roles? policy->num_roles : policy->num_role_trans * sizeof(bool_t));
 	if(b->trans == NULL) {
 		fprintf(stderr, "out of memory\n");
 		return -1;
 	}
-	memset(b->trans, 0, policy->num_role_trans * sizeof(bool_t));
+	memset(b->trans, 0, roles? policy->num_roles : policy->num_role_trans * sizeof(bool_t));
 	b->t_cnt = 0;
 	
 	return 0;
@@ -417,6 +422,29 @@ int add_i_to_a(int i, int *cnt, int **a)
 	return 0;
 }
 
+int add_uint_to_a(uint32_t i, uint32_t *cnt, uint32_t **a)
+{
+        if(cnt == NULL || a == NULL)
+                return -1;
+
+        /* FIX: This is not very elegant! We use an array that we
+         * grow as new int are added to an array.  But rather than be smart
+         * about it, for now we realloc() the array each time a new int is added! */
+        if(*a != NULL) {
+                *a = (uint32_t *) realloc(*a, (*cnt + 1) * sizeof(uint32_t));
+        } else /* empty list */ {
+                *cnt = 0;
+                *a = (uint32_t *) malloc(sizeof(uint32_t));
+        }
+        if(*a == NULL) {
+                fprintf(stderr, "out of memory\n");
+                return -1;
+        }
+        (*a)[*cnt] = i;
+        (*cnt)++;
+        return 0;
+}
+
 /* See if provided integer is in the provided integer array; if found return
  * the index for a, otherwise return -1 */
 int find_int_in_array(int i, const int *a, int a_sz)
@@ -454,6 +482,20 @@ int copy_int_array(int **dest, int *src, int len)
 	return 0;
 }
 
+int int_compare(const void *aptr, const void *bptr)
+{
+	int *a = (int*)aptr;
+	int *b = (int*)bptr;
+	
+	assert(a);
+	assert(b);
+
+	if (*a < *b)
+		return -1;
+	if (*a > *b)
+		return 1;
+	return 0;
+}
 
 bool_t is_name_in_list(const char *name, 
 			ta_item_t *list,
@@ -547,7 +589,7 @@ char* find_file(const char *file_name)
 			fprintf(stderr, "out of memory");
 			return NULL;
 		}
-		sprintf(dir, "./");
+		sprintf(dir, ".");
 		free(file);
 		return dir;
 	}
@@ -651,16 +693,15 @@ char *get_config_var(const char *var, FILE *fp)
 		line_ptr = &line[0];
 		if (trim_string(&line_ptr) != 0)
 			return NULL;
-		if(line[0] == '#' || sscanf(line, "%s %s", t1, t2) != 2 || strcasecmp(var, t1) != 0) {
+		if (line[0] == '#' || sscanf(line, "%s %[^\n]", t1, t2) != 2 || strcasecmp(var, t1) != 0) {
 			continue;
 		}
 		else {
-			result = (char *)malloc(strlen(t2) + 1);
-			if(result == NULL) {
+			result = (char *)malloc(sizeof(char) * (strlen(t2) + 1));
+			if (result == NULL) {
 				fprintf(stderr, "out of memory\n");
 				return NULL;
-			} 
-			else {
+			} else {
 				strcpy(result, t2);
 				return result;
 			}
@@ -698,32 +739,58 @@ char *config_var_list_to_string(const char **list, int size)
 char **get_config_var_list(const char *var, FILE *file, int *list_sz)
 {
 	char *values, *token;
-	char ** results = NULL;
-
+	char **results = NULL, **ptr = NULL;
+	int i; 
+	
+	assert(var != NULL || file != NULL || list_sz != NULL);
 	*list_sz = 0;
-	if ((values = get_config_var(var, file)) != NULL) {
-		token = strtok(values, ":");
-		while (token) {
-			(*list_sz)++;
-			results = (char**)realloc(results, sizeof(char*) * (*list_sz));
-			results[(*list_sz)-1] = (char*)malloc(sizeof(char) * (1+strlen(token)));
-			strcpy(results[(*list_sz)-1], token);
-			token = strtok(NULL, ":");
-		}
+	values = get_config_var(var, file);
+	if (values != NULL) {
+		while ((token = strsep(&values, ":")) != NULL) {
+		       	if (strcmp(token, "") && !str_is_only_white_space(token)) {
+		       		ptr = (char**)realloc(results, sizeof(char*) * (*list_sz + 1));
+				if (ptr == NULL) {
+					fprintf(stderr, "Out of memory.\n");
+					free(values);
+					/* If realloc fails, it will not free the 
+					 * original pointer, so we handle this here. */
+					if (results) {
+						for (i = 0; i < *list_sz; i++) 
+							free(results[i]);
+						free(results);
+					}
+					return NULL;
+				}
+				results = ptr;
+				(*list_sz)++;
+				/* Add 1 to include enough space for terminating null char */
+				results[(*list_sz) - 1] = (char*)malloc(sizeof(char) * (1 + strlen(token)));
+				if (results[(*list_sz) - 1] == NULL) {
+					fprintf(stderr, "Out of memory.\n");
+					free(values);
+					if (results) {
+						/* Free list up to the previous list item */
+						for (i = 0; i < *list_sz; i++) 
+							free(results[i]);
+						free(results);
+					}
+					return NULL;
+				}
+				strcpy(results[(*list_sz) - 1], token);
+	       	       	}
+	        }
 		free(values);
-		return results;
 	}
-	return NULL;
+	return results;
 }
 
 /* append a string to an existing string, expanding the target string if 
  * necessary.  Call must free the target string.  Make tgt == NULL if 
  * this is first use.
  */
-#define RE_STR_ALLOC_SZ	128
 int append_str(char **tgt, int *tgt_sz, const char *str)
 {
-	int str_len, tgt_len;
+	int str_len;
 	if(str == NULL || (str_len = strlen(str)) == 0)
 		return 0;
 	if(tgt == NULL)
@@ -731,22 +798,18 @@ int append_str(char **tgt, int *tgt_sz, const char *str)
 	str_len++;
 	/* target is currently empty */
 	if(*tgt == NULL) {
-		if(str_len < RE_STR_ALLOC_SZ)
-			str_len = RE_STR_ALLOC_SZ;
 		*tgt = (char *)malloc(str_len);
 		*tgt_sz = str_len;
 		strcpy(*tgt, str);
 		return 0;
-	}
+	} else {
 	/* tgt has some memory */
-	tgt_len = strlen(*tgt);
-	if(!(*tgt_sz >= (str_len + tgt_len))) {
 		*tgt = (char *)realloc(*tgt, *tgt_sz + str_len);
 		if(*tgt == NULL) {
 			fprintf(stderr, "out of memory");
 			return -1;
 		}
-		*tgt_sz = str_len + tgt_len;
+		*tgt_sz += str_len;
 	}
 	strcat(*tgt, str);
 	return 0;	
