@@ -1,9 +1,10 @@
-/* Copyright (C) 2003 Tresys Technology, LLC
+/* Copyright (C) 2003-2005 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
- * Author: Karl MacMillan <kmacmillan@tresys.com> and 
+ * Author: Karl MacMillan <kmacmillan@tresys.com> 
  *         Kevin Carr <kcarr@tresys.com>
+ *         Jeremy Stitz <jstitz@tresys.com>
  */
 
 #include "auditlogmodel.h"
@@ -294,9 +295,9 @@ static void log_view_store_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter
 		if (AVC_MISC_FIELD == column) {
 			string = g_string_new("");
 			policy_msg = store->log_view->my_log->msg_list[indx]->msg_data.load_policy_msg;
-			g_string_printf(string, "users=%d roles=%d types=%d classes=%d rules=%d",
+			g_string_printf(string, "users=%d roles=%d types=%d bools=%d classes=%d rules=%d",
 					policy_msg->users, policy_msg->roles, policy_msg->types,
-					policy_msg->classes, policy_msg->rules);
+					policy_msg->bools, policy_msg->classes, policy_msg->rules);
 			set_utf8_return_value(value, string->str);
 			g_string_free(string, TRUE);
 			return;
@@ -389,6 +390,8 @@ static void log_view_store_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter
 			return;
 		if (cur_msg->dev)
 			g_string_append_printf(string, "dev=%s ", cur_msg->dev);
+		if (cur_msg->ipaddr)
+		        g_string_append_printf(string, "ipaddr=%s ", cur_msg->ipaddr);
 		if (cur_msg->laddr)
 			g_string_append_printf(string, "laddr=%s ", cur_msg->laddr);
 		if (cur_msg->lport!=0)
@@ -413,6 +416,10 @@ static void log_view_store_get_value(GtkTreeModel *tree_model, GtkTreeIter *iter
 			g_string_append_printf(string, "key=%d ", cur_msg->key);
 		if (cur_msg->is_capability)
 			g_string_append_printf(string, "capability=%d ", cur_msg->capability);	
+		if (!(cur_msg->tm_stmp_sec == 0 && cur_msg->tm_stmp_nano == 0 && cur_msg->serial == 0)) {
+			g_string_append_printf(string, "timestamp=%lu.%03lu ", cur_msg->tm_stmp_sec, cur_msg->tm_stmp_nano);
+			g_string_append_printf(string, "serial=%u ", cur_msg->serial);
+		}
 		set_utf8_return_value(value, string->str);
 		g_string_free(string, TRUE);
 	};
@@ -661,21 +668,6 @@ static gboolean seaudit_log_view_store_has_default_sort_func (GtkTreeSortable *s
 	return FALSE;
 }
 
-static int int_compare(const void *aptr, const void *bptr)
-{
-	int *a = (int*)aptr;
-	int *b = (int*)bptr;
-	
-	assert(a);
-	assert(b);
-
-	if (*a < *b)
-		return -1;
-	if (*a > *b)
-		return 1;
-	return 0;
-}
-
 void seaudit_log_view_store_do_filter(SEAuditLogViewStore *store)
 {
 	int *deleted=NULL, num_deleted, num_kept, old_sz, new_sz, cnt=0, i;
