@@ -1,14 +1,16 @@
-/* Copyright (C) 2004 Tresys Technology, LLC
+/* Copyright (C) 2004-2005 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
  * Author: Kevin Carr <kcarr@tresys.com>
  * Date: December 31, 2003
+ * Modified: don.patterson@tresys.com 10-2004
  */
 
 #include "preferences.h"
 #include "utilgui.h"
 #include "seaudit.h"
+#include <glib/gprintf.h>
 
 extern seaudit_t *seaudit_app;
 
@@ -18,33 +20,64 @@ static void on_browse_policy_button_clicked(GtkWidget *widget, gpointer user_dat
 static void on_browse_log_button_clicked(GtkWidget *widget, gpointer user_data);
 
 
-void set_seaudit_conf_default_policy(seaudit_conf_t *conf, const char *filename)
+int set_seaudit_conf_default_policy(seaudit_conf_t *conf, const char *filename)
 {
 	if (conf->default_policy_file)
 		free(conf->default_policy_file);
 	if (filename) {
 		conf->default_policy_file = (char*)malloc(sizeof(char) * (1 + strlen(filename)));
+		if (conf->default_policy_file == NULL) {
+			fprintf(stderr, "Out of memory.\n");	
+			return -1;
+		}
 		strcpy(conf->default_policy_file, filename);
 	} else 
 		conf->default_policy_file = NULL;
+		
+	return 0;
 }
 
-void set_seaudit_conf_default_log(seaudit_conf_t *conf, const char *filename)
+int set_seaudit_conf_default_log(seaudit_conf_t *conf, const char *filename)
 {
 	if (conf->default_log_file)
 		free(conf->default_log_file);
 
 	if (filename) {
 		conf->default_log_file = (char*)malloc(sizeof(char) * (1 + strlen(filename)));
+		if (conf->default_log_file == NULL) {
+			fprintf(stderr, "Out of memory.\n");	
+			return -1;
+		}
 		strcpy(conf->default_log_file, filename);
 	} else
 		conf->default_log_file = NULL;
+		
+	return 0;
+}
+
+static int set_seaudit_conf_file_path(char **conf_variable, const char *filename)
+{
+	assert(conf_variable != NULL);
+	if (*conf_variable)
+		free(*conf_variable);
+
+	if (filename) {
+		*conf_variable = (char*)malloc(sizeof(char) * (1 + strlen(filename)));
+		if (*conf_variable == NULL) {
+			fprintf(stderr, "Out of memory.\n");	
+			return -1;
+		}
+		strcpy(*conf_variable, filename);
+	} else
+		*conf_variable = NULL;
+		
+	return 0;
 }
 
 int load_seaudit_conf_file(seaudit_conf_t *conf)
 {
   	FILE *file;
-	int i, size, index;
+	int i, j, size, index;
 	GString *path;
 	char *value, **list, *dir;
 
@@ -71,33 +104,64 @@ int load_seaudit_conf_file(seaudit_conf_t *conf)
 	if (!file)
 		return -1; 
 	value = get_config_var("DEFAULT_LOG_FILE", file);
-	set_seaudit_conf_default_log(conf, value);
+	if (set_seaudit_conf_default_log(conf, value) != 0) 
+		goto err;	
+
 	if (value)
 		free(value);
 	value = get_config_var("DEFAULT_POLICY_FILE", file);
-	set_seaudit_conf_default_policy(conf, value);
+	if (set_seaudit_conf_default_policy(conf, value) != 0) 
+		goto err;	
+
 	if (value)
 		free(value);
+	
+	value = get_config_var("DEFAULT_REPORT_CONFIG_FILE", file);
+	if (set_seaudit_conf_file_path(&conf->default_seaudit_report_config_file, value) != 0) 
+		goto err;	
+
+	if (value)
+		free(value);
+		
+	value = get_config_var("DEFAULT_REPORT_CSS_FILE", file);
+	if (set_seaudit_conf_file_path(&conf->default_seaudit_report_css_file, value) != 0) 
+		goto err;	
+
+	if (value)
+		free(value);
+			
 	list = get_config_var_list("RECENT_LOG_FILES", file, &size);
 	if (list) {
 		for (i = 0; i < size; i++) {
-			add_path_to_recent_log_files(list[i], conf);
+			if (add_path_to_recent_log_files(list[i], conf) != 0) {
+				/* Free the remaining paths that exist in the list */
+				for (j = i; j < size; j++)
+					free(list[j]);
+				free(list);
+				goto err;
+			}
 			free(list[i]);
 		}
 		free(list);
 	} else 
 		conf->recent_log_files = NULL;
-	conf->num_recent_log_files = size;
+		
 	list = get_config_var_list("RECENT_POLICY_FILES", file, &size);
 	if (list) {
 		for (i = 0; i < size; i++) {
-			add_path_to_recent_policy_files(list[i], conf);
+			if (add_path_to_recent_policy_files(list[i], conf) != 0) {
+				/* Free the remaining paths that exist in the list */
+				for (j = i; j < size; j++)
+					free(list[j]);
+				free(list);
+				goto err;
+			}
 			free(list[i]);
 		}
 		free(list);
 	} else
 		conf->recent_policy_files = NULL;
-	conf->num_recent_policy_files = size;
+	
 	for (i = 0; i < NUM_FIELDS; i++)
 		conf->column_visibility[i] = TRUE;
 	list = get_config_var_list("LOG_COLUMNS_HIDDEN", file, &size);
@@ -118,64 +182,101 @@ int load_seaudit_conf_file(seaudit_conf_t *conf)
 		conf->real_time_log = atoi(value);
 		free(value);
 	}
+	value = get_config_var("REAL_TIME_LOG_UPDATE_INTERVAL", file);
+	if (!value)
+		conf->real_time_interval = DEFAULT_LOG_UPDATE_INTERVAL;
+	else  {
+		conf->real_time_interval = atoi(value);
+		free(value);
+	}
 	fclose(file);
+
 	return 0;
+err:
+	free_seaudit_conf(conf);
+	return -1;
 }
 
-void add_path_to_recent_log_files(const char *path, seaudit_conf_t *conf_file)
+int add_path_to_recent_log_files(const char *path, seaudit_conf_t *conf_file)
 {
 	int i;
-
-	if (path == NULL || conf_file == NULL)
-		return;
+	char **ptr = NULL;
+	
+	assert(path != NULL || conf_file != NULL);
 
 	/* make sure we don't add duplicates */
 	for (i = 0; i < conf_file->num_recent_log_files; i++)
 		if (strcmp(path, conf_file->recent_log_files[i]) == 0)
-			return;
+			return 0;
 	if (conf_file->num_recent_log_files >= 5) {
 		free(conf_file->recent_log_files[0]);
 		for (i = 1; i < conf_file->num_recent_log_files; i++)
-			conf_file->recent_log_files[i-1] = conf_file->recent_log_files[i];
-		conf_file->recent_log_files[conf_file->num_recent_log_files-1] = (char *)malloc(sizeof(char)*(strlen(path) + 1));
-		strcpy(conf_file->recent_log_files[conf_file->num_recent_log_files-1], path);
-		return;
-
+			conf_file->recent_log_files[i - 1] = conf_file->recent_log_files[i];
+		conf_file->recent_log_files[conf_file->num_recent_log_files - 1] = 
+			(char *)malloc(sizeof(char)*(strlen(path) + 1));
+		if (conf_file->recent_log_files[conf_file->num_recent_log_files - 1] == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		strcpy(conf_file->recent_log_files[conf_file->num_recent_log_files - 1], path);
 	} else {
-		conf_file->recent_log_files = (char**)realloc( conf_file->recent_log_files, sizeof(char*)*(conf_file->num_recent_log_files+1));
-		conf_file->recent_log_files[conf_file->num_recent_log_files] = (char *)malloc(sizeof(char)*(strlen(path) + 1));
-		strcpy(conf_file->recent_log_files[conf_file->num_recent_log_files], path);	
+		ptr = (char**)realloc(conf_file->recent_log_files, 
+			sizeof(char*)*(conf_file->num_recent_log_files + 1));
+		if (ptr == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		conf_file->recent_log_files = ptr;
 		conf_file->num_recent_log_files++;
-		return;
+		conf_file->recent_log_files[conf_file->num_recent_log_files - 1] = 
+			(char *)malloc(sizeof(char)*(strlen(path) + 1));
+		if (conf_file->recent_log_files[conf_file->num_recent_log_files - 1] == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		strcpy(conf_file->recent_log_files[conf_file->num_recent_log_files - 1], path);	
 	}
+	return 0;
 }
 
-void add_path_to_recent_policy_files(const char *path, seaudit_conf_t *conf_file)
+int add_path_to_recent_policy_files(const char *path, seaudit_conf_t *conf_file)
 {
 	int i;
+	char **ptr = NULL;
 
-	if (path == NULL || conf_file == NULL)
-		return;
+	assert(path != NULL || conf_file != NULL);
 
 	/* make sure we don't add duplicates */
 	for (i = 0; i < conf_file->num_recent_policy_files; i++)
 		if (strcmp(path, conf_file->recent_policy_files[i]) == 0)
-			return;
+			return 0;
 	if (conf_file->num_recent_policy_files >= 5) {
 		free(conf_file->recent_policy_files[0]);
 		for (i = 1; i < conf_file->num_recent_policy_files; i++)
-			conf_file->recent_log_files[i-1] = conf_file->recent_log_files[i];
-		conf_file->recent_policy_files[conf_file->num_recent_policy_files-1] = (char *)malloc(sizeof(char)*(strlen(path) + 1));
-		strcpy(conf_file->recent_policy_files[conf_file->num_recent_policy_files-1], path);
-		return;
+			conf_file->recent_policy_files[i - 1] = conf_file->recent_policy_files[i];
+		if (conf_file->recent_policy_files[conf_file->num_recent_policy_files - 1] == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		strcpy(conf_file->recent_policy_files[conf_file->num_recent_policy_files - 1], path);
 	} else {
-		conf_file->recent_policy_files = (char**)realloc(conf_file->recent_policy_files, 							  
-							 sizeof(char*)*(conf_file->num_recent_policy_files+1));
-		conf_file->recent_policy_files[conf_file->num_recent_policy_files] = (char *)malloc(sizeof(char)*(strlen(path) + 1));
-		strcpy(conf_file->recent_policy_files[conf_file->num_recent_policy_files], path);
+		ptr = (char**)realloc(conf_file->recent_policy_files, 							  
+			sizeof(char*)*(conf_file->num_recent_policy_files + 1));
+		if (ptr == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		conf_file->recent_policy_files = ptr;
 		conf_file->num_recent_policy_files++;
-		return;
+		conf_file->recent_policy_files[conf_file->num_recent_policy_files - 1] = 
+			(char *)malloc(sizeof(char)*(strlen(path) + 1));
+		if (conf_file->recent_policy_files[conf_file->num_recent_policy_files - 1] == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			return -1;
+		}
+		strcpy(conf_file->recent_policy_files[conf_file->num_recent_policy_files - 1], path);	
 	}
+	return 0;
 }
 
 int save_seaudit_conf_file(seaudit_conf_t *conf)
@@ -210,6 +311,16 @@ int save_seaudit_conf_file(seaudit_conf_t *conf)
 		fprintf(file, " %s\n", conf->default_policy_file);
 	else 
 		fprintf(file, "\n");
+	fprintf(file, "DEFAULT_REPORT_CONFIG_FILE");
+	if (conf->default_seaudit_report_config_file)
+		fprintf(file, " %s\n", conf->default_seaudit_report_config_file);
+	else 
+		fprintf(file, "\n");
+	fprintf(file, "DEFAULT_REPORT_CSS_FILE");
+	if (conf->default_seaudit_report_css_file)
+		fprintf(file, " %s\n", conf->default_seaudit_report_css_file);
+	else 
+		fprintf(file, "\n");
 	fprintf(file, "RECENT_LOG_FILES");
 	value = config_var_list_to_string((const char**)conf->recent_log_files, conf->num_recent_log_files);
 	if (value) {
@@ -247,7 +358,8 @@ int save_seaudit_conf_file(seaudit_conf_t *conf)
 		}
 	}
 		
-	fprintf(file, "\nREAL_TIME_LOG_MONITORING %d\n", conf->real_time_log);
+	fprintf(file, "\nREAL_TIME_LOG_MONITORING %d", conf->real_time_log);
+	fprintf(file, "\nREAL_TIME_LOG_UPDATE_INTERVAL %d\n", conf->real_time_interval);
 	fclose(file);
 	return 0;
 }
@@ -288,24 +400,64 @@ void update_column_visibility(seaudit_filtered_view_t *view, gpointer user_data)
 	}	
 }
 
+static void change_log_update_interval(seaudit_conf_t *conf_file, int millisecs)
+{
+	assert(millisecs > 0);
+	conf_file->real_time_interval =  millisecs;
+}
+
 void on_prefer_window_ok_button_clicked(GtkWidget *widget, gpointer user_data)
 {
 	GtkWidget *prefer_window;
 	GladeXML *xml = (GladeXML*)user_data;
-	GtkEntry *log_entry, *pol_entry;
-
+	GtkEntry *log_entry, *pol_entry, *report_css, *report_config, *interval_lbl;
+	seaudit_conf_t *seaudit_conf = NULL;
+	const gchar *interval_str = NULL;
+	int interval;
+	
 	prefer_window = glade_xml_get_widget(xml, "PreferWindow");
 	g_assert(widget);
 	log_entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultLogEntry"));
 	g_assert(log_entry);
 	pol_entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultPolicyEntry"));
 	g_assert(pol_entry);
-	set_seaudit_conf_default_log(&(seaudit_app->seaudit_conf),
-				     gtk_entry_get_text(log_entry));
-	set_seaudit_conf_default_policy(&(seaudit_app->seaudit_conf),
-					gtk_entry_get_text(pol_entry));
-	save_seaudit_conf_file(&(seaudit_app->seaudit_conf));
+	report_css = GTK_ENTRY(glade_xml_get_widget(xml, "report-css-entry"));
+	g_assert(report_css);
+	report_config = GTK_ENTRY(glade_xml_get_widget(xml, "report-config-entry"));
+	g_assert(report_config);
+	interval_lbl = GTK_ENTRY(glade_xml_get_widget(xml, "interval_lbl"));
+	g_assert(interval_lbl);
+	
+	seaudit_conf = &(seaudit_app->seaudit_conf);
+	interval_str = gtk_entry_get_text(interval_lbl);
+	if (!str_is_only_white_space(interval_str)) {
+		interval = atoi(interval_str);
+		if (interval > 0) 
+			change_log_update_interval(seaudit_conf, interval);
+		else {
+			message_display(seaudit_app->window->window, 
+				GTK_MESSAGE_ERROR, "Update interval must be greater than 0!");
+			return;
+		}
+	} else {
+		message_display(seaudit_app->window->window, 
+			GTK_MESSAGE_ERROR, "Update interval cannot be empty!");
+		return;
+	}
 
+	set_seaudit_conf_default_log(seaudit_conf,
+				     gtk_entry_get_text(log_entry));
+	set_seaudit_conf_default_policy(seaudit_conf,
+					gtk_entry_get_text(pol_entry));
+	
+	if (set_seaudit_conf_file_path(&(seaudit_conf->default_seaudit_report_config_file), 
+		gtk_entry_get_text(report_config)) != 0) 
+		return;
+	if (set_seaudit_conf_file_path(&(seaudit_conf->default_seaudit_report_css_file), 
+		gtk_entry_get_text(report_css)) != 0) 
+		return;
+	save_seaudit_conf_file(seaudit_conf);
+	
 	/* set the updated visibility if needed */
 	if (!seaudit_app->column_visibility_changed)
 		return;
@@ -314,53 +466,17 @@ void on_prefer_window_ok_button_clicked(GtkWidget *widget, gpointer user_data)
 	gtk_widget_destroy(prefer_window);
 }
 
-static void on_browse_log_button_clicked(GtkWidget *widget, gpointer user_data)
+static void display_browse_dialog_for_entry_box(GtkEntry *entry, const char *file_path, const char *title)
 {
-	GladeXML *xml = (GladeXML*)user_data;
-	GtkEntry *entry;
 	GtkWidget *file_selector;
 	gint response;
 	const gchar *filename;
 
-	entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultLogEntry"));
 	g_assert(entry);
-	file_selector = gtk_file_selection_new("Select Default Log");
+	file_selector = gtk_file_selection_new(title);
 	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
-	if (seaudit_app->seaudit_conf.default_policy_file != NULL)
-		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), gtk_entry_get_text(GTK_ENTRY(entry)));
-	g_signal_connect(GTK_OBJECT(file_selector), "response", 
-			 G_CALLBACK(get_dialog_response), &response);
-	while (1) {
-		gtk_dialog_run(GTK_DIALOG(file_selector));
-		if (response != GTK_RESPONSE_OK) {
-
-			gtk_widget_destroy(file_selector);
-			return;
-		}
-		filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
-		if (g_file_test(filename, G_FILE_TEST_EXISTS) && !g_file_test(filename, G_FILE_TEST_IS_DIR)) 
-			break;
-		if (g_file_test(filename, G_FILE_TEST_IS_DIR))
-			gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), filename);
-	}
-	gtk_entry_set_text(GTK_ENTRY(entry), filename);
-	gtk_widget_destroy(file_selector);
-}
-
-static void on_browse_policy_button_clicked(GtkWidget *widget, gpointer user_data)
-{
-	GladeXML *xml = (GladeXML*)user_data;
-	GtkEntry *entry;
-	GtkWidget *file_selector;
-	gint response;
-	const gchar *filename;
-
-	entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultPolicyEntry"));
-	g_assert(entry);
-	file_selector = gtk_file_selection_new("Select Default Policy");
-	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selector));
-	if (seaudit_app->seaudit_conf.default_policy_file != NULL)
-		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), gtk_entry_get_text(GTK_ENTRY(entry)));
+	if (file_path != NULL)
+		gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), gtk_entry_get_text(entry));
 	g_signal_connect(GTK_OBJECT(file_selector), "response", 
 			 G_CALLBACK(get_dialog_response), &response);
 	while (1) {
@@ -375,10 +491,52 @@ static void on_browse_policy_button_clicked(GtkWidget *widget, gpointer user_dat
 		if (g_file_test(filename, G_FILE_TEST_IS_DIR))
 			gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector), filename);
 	}
-	gtk_entry_set_text(GTK_ENTRY(entry), filename);
+	gtk_entry_set_text(entry, filename);
 	gtk_widget_destroy(file_selector);
 }
 
+static void on_browse_log_button_clicked(GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML*)user_data;
+	GtkEntry *entry;
+
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultLogEntry"));
+	display_browse_dialog_for_entry_box(entry, 
+		seaudit_app->seaudit_conf.default_log_file, "Select Default Log");
+}
+
+static void on_browse_policy_button_clicked(GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML*)user_data;
+	GtkEntry *entry;
+
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultPolicyEntry"));
+	display_browse_dialog_for_entry_box(entry, 
+		seaudit_app->seaudit_conf.default_policy_file, "Select Default Policy");
+}
+
+static void on_browse_report_css_button_clicked(GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML*)user_data;
+	GtkEntry *entry;
+
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "report-css-entry"));
+	display_browse_dialog_for_entry_box(entry, 
+		seaudit_app->seaudit_conf.default_seaudit_report_css_file, 
+		"Select HTML Report Style Sheet File");
+}
+
+static void on_browse_report_config_button_clicked(GtkWidget *widget, gpointer user_data)
+{
+	GladeXML *xml = (GladeXML*)user_data;
+	GtkEntry *entry;
+
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "report-config-entry"));
+	display_browse_dialog_for_entry_box(entry, 
+		seaudit_app->seaudit_conf.default_seaudit_report_config_file, 
+		"Select Report Configuration File");
+}
+
 static void on_preference_toggled(GtkToggleButton *toggle, gpointer user_data)
 {
 	if (!strcmp("MessageCheck", gtk_widget_get_name(GTK_WIDGET(toggle)))) {
@@ -458,7 +616,9 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 	GtkToggleButton *toggle = NULL;
 	GString *path;
 	char *dir;
-
+	GString *interval = g_string_new("");
+	
+	assert(interval);
 	dir = find_file("prefer_window.glade");
 	if (!dir){
 		fprintf(stderr, "could not find prefer_window.glade\n");
@@ -471,7 +631,13 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 	g_string_free(path, TRUE);
 	window = glade_xml_get_widget(xml, "PreferWindow");
 	g_assert(window);
-
+	
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "interval_lbl"));
+	g_assert(entry);
+	g_string_printf(interval, "%d", seaudit_app->seaudit_conf.real_time_interval);
+	assert(interval != NULL);
+	gtk_entry_set_text(entry, interval->str);
+		
 	entry = GTK_ENTRY(glade_xml_get_widget(xml, "DefaultLogEntry"));
 	g_assert(entry);
 	if (seaudit_app->seaudit_conf.default_log_file)
@@ -481,6 +647,16 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 	g_assert(entry);
 	if (seaudit_app->seaudit_conf.default_policy_file)
 		gtk_entry_set_text(entry, seaudit_app->seaudit_conf.default_policy_file);
+	
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "report-config-entry"));
+	g_assert(entry);
+	if (seaudit_app->seaudit_conf.default_seaudit_report_config_file)
+		gtk_entry_set_text(entry, seaudit_app->seaudit_conf.default_seaudit_report_config_file);
+		
+	entry = GTK_ENTRY(glade_xml_get_widget(xml, "report-css-entry"));
+	g_assert(entry);
+	if (seaudit_app->seaudit_conf.default_seaudit_report_css_file)
+		gtk_entry_set_text(entry, seaudit_app->seaudit_conf.default_seaudit_report_css_file);
 
 	button = glade_xml_get_widget(xml, "OkButton");
 	g_assert(button);
@@ -502,6 +678,20 @@ void on_preferences_activate(GtkWidget *widget, GdkEvent *event, gpointer callba
 			  "clicked",
 			  G_CALLBACK (on_browse_policy_button_clicked),
 			  (gpointer) xml);
+		
+	button = glade_xml_get_widget(xml, "report-css-button");
+	g_assert(widget);
+	g_signal_connect (GTK_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (on_browse_report_css_button_clicked),
+			  (gpointer) xml);
+	
+	button = glade_xml_get_widget(xml, "report-config-button");
+	g_assert(widget);
+	g_signal_connect (GTK_OBJECT (button),
+			  "clicked",
+			  G_CALLBACK (on_browse_report_config_button_clicked),
+			  (gpointer) xml);
 
 	glade_xml_signal_connect(xml, "on_preference_toggled", G_CALLBACK(on_preference_toggled));
 
