@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004 Tresys Technology, LLC
+ *  Copyright (C) 2003-2005 Tresys Technology, LLC
  *  see file 'COPYING' for use and warranty information
  *
  */
@@ -12,6 +12,8 @@
  *  replcon: a tool for replacing file contexts in SE Linux
  */
 
+#include <fsdata.h>
+
 /* SE Linux includes*/
 #include <selinux/selinux.h>
 #include <selinux/context.h>
@@ -32,6 +34,10 @@
 #include <ftw.h>
 #include <mntent.h>
 
+#include <policy.h>
+
+
+
 /* REPLCON_VERSION_NUM should be defined in the make environment */
 #ifndef REPLCON_VERSION_NUM
 #define REPLCON_VERSION_NUM "UNKNOWN"
@@ -42,45 +48,15 @@
 #define FINDCON_VERSION_NUM "UNKNOWN"
 #endif
 
-#define COPYRIGHT_INFO "Copyright (C) 2004 Tresys Technology, LLC"
+#define COPYRIGHT_INFO "Copyright (C) 2003-2005 Tresys Technology, LLC"
 
 #define DEBUG 0
-#define NUM_OBJECT_CLASSES 8
 #define MAX_INPUT_SIZE 1024
 #define NFTW_FLAGS FTW_MOUNT | FTW_PHYS
 #define NFTW_DEPTH 1024
 
-#ifndef TRUE
-#define TRUE 1
-#endif
-
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef SUPPORTED_FILESYSTEMS
-#define SUPPORTED_FILESYSTEMS "ext2 ext3"
-#endif
-
-#ifndef UNLABELED
-#define UNLABELED "UNLABELED"
-#endif
 
 /* Data Structures */
-typedef enum replcon_classes {
-	NORM_FILE,
-	DIR,
-	LNK_FILE,
-	CHR_FILE,
-	BLK_FILE,
-	SOCK_FILE,
-	FIFO_FILE,
-	ALL_FILES
-} replcon_classes_t;
-
-const char *replcon_object_classes[] =
-    { "file", "dir", "lnk_file", "chr_file", "blk_file", "sock_file",
-"fifo_file", "all_files" };
 
 typedef struct replcon_context {
 	char *user;
@@ -99,7 +75,7 @@ typedef struct replcon_info {
 	unsigned char quiet;
 	unsigned char stdin;
 	unsigned char unlabeled;
-	replcon_classes_t *obj_classes;
+	sefs_classes_t *obj_classes;
 	int num_classes;
 #ifndef FINDCON
 	replcon_context_pair_t *pairs;
@@ -129,6 +105,15 @@ static struct option const longopts[] = {
 	{NULL, 0, NULL, 0}
 };
 
+
+static void sefs_double_array_print(char **array,int size)
+{
+	int i;
+	for (i=0;i<size;i++){
+		printf("%s\n",array[i]);
+	}
+}
+
 /*
  * replcon_context_free
  *
@@ -316,31 +301,7 @@ get_security_context(const replcon_context_t *context)
 	return sec_con;
 }
 
-/*
- * replcon_print_valid_object_classes
- *
- * Prints out the valid object classes to specify for the search.
- */
-void
-replcon_print_valid_object_classes()
-{
-	int i, num_objs_on_line = 0, obj_max = 8;
-	
-	assert(replcon_object_classes != NULL);
-	printf("   ");
-	for (i = 0; i < NUM_OBJECT_CLASSES; i++) {
-		num_objs_on_line++;
-		if (i == (NUM_OBJECT_CLASSES - 1))
-			printf("%s\n", replcon_object_classes[i]);
-		else if (num_objs_on_line == obj_max) {
-			printf("%s,\n", replcon_object_classes[i]);
-			printf("   ");
-		}
-		else 
-			printf("%s, ", replcon_object_classes[i]);
-		
-	}
-}
+
 
 /*
  * replcon_usage
@@ -351,6 +312,8 @@ replcon_print_valid_object_classes()
 void
 replcon_usage(const char *program_name, int brief)
 {
+	char **array=NULL;
+	int size;
 #ifndef FINDCON
 	printf("%s (replcon ver. %s)\n\n", COPYRIGHT_INFO, REPLCON_VERSION_NUM);
 	printf("Usage: %s [OPTIONS] -c OLD NEW FILENAMES\n", program_name);
@@ -379,11 +342,17 @@ replcon_usage(const char *program_name, int brief)
 	printf("  -h,  --help             Display this help and exit.\n");
 	printf("\n");
 	printf("A context may be specified as a colon separated list of user, role, and type\n");
-	printf("as follows - user_u:object_r:user_t.  replcon will automatically match a user,\n");
+	printf("as follows - user_u:object_r:user_t. The tool will automatically match a user,\n");
 	printf("role, or type that is not specified, with any other user, role, or type.\n");
-	printf("For example ::user_t specifies any context that has user_t as the type.\n\n");
+	printf("For example ::user_t specifies any context that has user_t as the type.\n");
+	printf("\nThe special string 'unlabeled' can be provided to the -c option in order\n");
+	printf("to find or replace files that have no label.\n\n");
+	
 	printf("Valid OBJECT classes to specify include: \n");
-	replcon_print_valid_object_classes();
+	array = sefs_get_valid_object_classes(&size);
+	sefs_double_array_print(array,size);
+	sefs_double_array_destroy(array,size);
+				
 	printf("\n");
 }
 
@@ -465,34 +434,18 @@ replcon_info_free(replcon_info_t *info)
  *
  * Check if replcon_info has an object class
  */
-int replcon_info_has_object_class(replcon_info_t *info, replcon_classes_t obj_class)
+int replcon_info_has_object_class(replcon_info_t *info, sefs_classes_t obj_class)
 {
 	int i;
 
 	assert(info != NULL); 
 	for (i = 0; i < info->num_classes; i++)
 		if (info->obj_classes[i] == obj_class
-		    || replcon_info.obj_classes[i] == ALL_FILES)
+		    || replcon_info.obj_classes[i] == SEFS_ALL_FILES)
 			return TRUE;
 	return FALSE;
 }
 
-/*
- * replcon_is_valid_object_class
- *
- * Determines if class_name is a valid object class.  Return -1 if invalid
- * otherwise the index of the valid object class
- */
-int replcon_is_valid_object_class(const char *class_name)
-{
-	int i;
-	
-	assert(class_name != NULL);
-	for (i = 0; i < NUM_OBJECT_CLASSES; i++)
-		if (strcmp(class_name, replcon_object_classes[i]) == 0)
-			return i;
-	return -1;
-}
 
 /*
  * replcon_is_valid_context_format
@@ -520,30 +473,6 @@ int replcon_is_valid_context_format(const char *context_str)
 		return FALSE;
 }
 
-/*
- * replcon_get_file_class
- *
- * Determines the file's class, and returns it
- */
-int replcon_get_file_class(const struct stat *statptr)
-{
-	assert(statptr != NULL);
-	if (S_ISREG(statptr->st_mode))
-		return NORM_FILE;
-	if (S_ISDIR(statptr->st_mode))
-		return DIR;
-	if (S_ISLNK(statptr->st_mode))
-		return LNK_FILE;
-	if (S_ISCHR(statptr->st_mode))
-		return CHR_FILE;
-	if (S_ISBLK(statptr->st_mode))
-		return BLK_FILE;
-	if (S_ISSOCK(statptr->st_mode))
-		return SOCK_FILE;
-	if (S_ISFIFO(statptr->st_mode))
-		return FIFO_FILE;
-	return ALL_FILES;
-}
 
 /*
  * replcon_info_add_object_class
@@ -553,10 +482,36 @@ int replcon_get_file_class(const struct stat *statptr)
  */
 int replcon_info_add_object_class(replcon_info_t *info, const char *str)
 {
-	replcon_classes_t class_id;
+	sefs_classes_t class_id;
 
 	assert(info != NULL);
-	class_id = replcon_is_valid_object_class(str);
+	class_id = sefs_is_valid_object_class(str);
+	switch(class_id) {
+	case 0:
+		class_id = SEFS_NORM_FILE;
+		break;
+	case 1:
+		class_id = SEFS_DIR;
+		break;
+	case 2:
+		class_id = SEFS_LNK_FILE;
+		break;
+	case 3:
+		class_id = SEFS_CHR_FILE;
+		break;
+	case 4:
+		class_id = SEFS_BLK_FILE;
+		break;
+	case 5:
+		class_id = SEFS_SOCK_FILE;
+		break;
+	case 6:
+		class_id = SEFS_FIFO_FILE;
+		break;
+	default:
+		class_id = SEFS_ALL_FILES;
+		break;
+ 	}
 
 	/* Check the object class */
 	if (class_id == -1) {
@@ -565,8 +520,8 @@ int replcon_info_add_object_class(replcon_info_t *info, const char *str)
 	}
 
 	info->obj_classes =
-	    (replcon_classes_t *) realloc(info->obj_classes,
-					  sizeof (replcon_classes_t) *
+	    (sefs_classes_t *) realloc(info->obj_classes,
+					  sizeof (sefs_classes_t) *
 					  (info->num_classes + 1));
 	if (!info->obj_classes) {
 		fprintf(stderr, "Error: Out of memory\n");
@@ -761,7 +716,7 @@ replcon_context_equal(const replcon_context_t *context, const replcon_context_t
  * The caller must pass a valid filename and statptr
  */
 int
-replcon_file_context_replace(const char *filename, const struct stat *statptr,
+replcon_file_context_replace(const char *filename, const struct stat64 *statptr,
 			     int fileflags, struct FTW *pfwt)
 {
 	int file_class, i;
@@ -772,7 +727,7 @@ replcon_file_context_replace(const char *filename, const struct stat *statptr,
 
 	assert(filename != NULL && statptr != NULL);
 	info = &replcon_info;
-	file_class = replcon_get_file_class(statptr);
+	file_class = sefs_get_file_class(statptr);
 	if (!replcon_info_has_object_class(info, file_class))
 		return 0;
 
@@ -857,7 +812,7 @@ replcon_file_context_replace(const char *filename, const struct stat *statptr,
  * The caller must pass a valid filename and statptr
  */
 int
-findcon(const char *filename, const struct stat *statptr,
+findcon(const char *filename, const struct stat64 *statptr,
 			     int fileflags, struct FTW *pfwt)
 {
 	int file_class, i;
@@ -865,7 +820,7 @@ findcon(const char *filename, const struct stat *statptr,
 	security_context_t file_con;
 	
 	assert(filename != NULL && statptr != NULL);
-	file_class = replcon_get_file_class(statptr);
+	file_class = sefs_get_file_class(statptr);
 	if (!replcon_info_has_object_class(&replcon_info, file_class))
 		return 0;
 
@@ -911,14 +866,14 @@ findcon(const char *filename, const struct stat *statptr,
 void
 replcon_stat_file_replace_context(const char *filename)
 {
-	struct stat file_status;
+	struct stat64 file_status;
 	/* Use path length limit defined in limits.h */
 	char actual_path[PATH_MAX+1];
 	char *ptr = NULL;
 	int i;
 
 	assert(filename != NULL);
-	if (stat(filename, &file_status) != 0) {
+	if (stat64(filename, &file_status) != 0) {
 		fprintf(stderr,
 			"Warning: Can not stat \'%s\'.  Skipping this file.\n",
 			filename);
@@ -931,13 +886,13 @@ replcon_stat_file_replace_context(const char *filename)
 			return;
 		}
 #ifndef FINDCON
-		if (nftw(actual_path, replcon_file_context_replace, NFTW_DEPTH, NFTW_FLAGS)) {
+		if (nftw64(actual_path, replcon_file_context_replace, NFTW_DEPTH, NFTW_FLAGS)) {
 			fprintf(stderr,
 				"Error walking directory tree: %s\n", actual_path);
 			return;
 		}
 #else
-		if (nftw(actual_path, findcon, NFTW_DEPTH, NFTW_FLAGS)) {
+		if (nftw64(actual_path, findcon, NFTW_DEPTH, NFTW_FLAGS)) {
 			fprintf(stderr,
 				"Error walking directory tree: %s\n", actual_path);
 			return;
@@ -946,13 +901,13 @@ replcon_stat_file_replace_context(const char *filename)
 		for(i = 0; i < num_mounts; i++) {
 			if (strstr(mounts[i], actual_path) == mounts[i])
 #ifndef FINDCON
-				if (nftw(mounts[i], replcon_file_context_replace, NFTW_DEPTH, NFTW_FLAGS)) {
+				if (nftw64(mounts[i], replcon_file_context_replace, NFTW_DEPTH, NFTW_FLAGS)) {
 					fprintf(stderr,
 						"Error walking directory tree: %s\n", mounts[i]);
 					return;
 				}
 #else
-				if (nftw(mounts[i], findcon, NFTW_DEPTH, NFTW_FLAGS)) {
+				if (nftw64(mounts[i], findcon, NFTW_DEPTH, NFTW_FLAGS)) {
 					fprintf(stderr,
 						"Error walking directory tree: %s\n", mounts[i]);
 					return;
@@ -1116,59 +1071,22 @@ int
 main(int argc, char **argv)
 {
 	char stream_input[MAX_INPUT_SIZE];
-	int i, len=10;
-	FILE *mtab = NULL;
-	struct mntent *entry;
-	char *token, *fs, *fs_orig = NULL;
+	int i, rw;
 
 	replcon_info_init(&replcon_info);
 	replcon_parse_command_line(argc, argv);
 	num_mounts = 0;
 
-	if ((mtab = fopen("/etc/mtab", "r")) == NULL) {
-		fprintf(stderr, "Could not open /etc/mtab for reading.\n");
-		goto err;
-	}
-
-	if ((mounts = malloc(sizeof(char*) * len)) == NULL) {
-		fprintf(stderr, "Out of memory.\n");
-		goto err;
-	}
-
-	while ((entry = getmntent(mtab))) {
 #ifndef FINDCON
-		if (hasmntopt(entry, MNTOPT_RW) == NULL)
-			continue;
+	rw = 0;
+#else
+	rw = 1;
 #endif
-		if(num_mounts >= len) {
-			len *= 2;
-			mounts = realloc(mounts, sizeof(char*) * len);
-			if (mounts == NULL) {
-				fprintf(stderr, "Out of memory.\n");
-				fclose(mtab);
-				goto err;
-			}
-		}
-		fs_orig = fs = strdup(SUPPORTED_FILESYSTEMS);
-		if (fs_orig == NULL) {
-			fprintf(stderr, "Out of memory.\n");
-			fclose(mtab);
-			goto err;
-		}
-		while((token = strtok(fs, " \t")) != NULL) {
-			if (fs) fs = NULL; /* for subsequent strtok calls */
-			if(!strcmp(token, entry->mnt_type)) {
-				if ((mounts[num_mounts++] = strdup(entry->mnt_dir)) == NULL) {
-					fprintf(stderr, "Out of memory.\n");
-					fclose(mtab);
-					goto err;
-				}
-				break;
-			}
-		}
-		free(fs_orig);
+
+	if (find_mount_points("/", &mounts, &num_mounts, rw)) {
+		fprintf(stderr, "Could not enumerate mountpoints.\n");
+		goto err;
 	}
-	fclose(mtab);
 
 	if (replcon_info.stdin) {
 		while (fgets(stream_input, (MAX_INPUT_SIZE - 1), stdin)) {
@@ -1191,6 +1109,5 @@ main(int argc, char **argv)
 err:
 	replcon_info_free(&replcon_info);
 	if (mounts) free(mounts);
-	if (fs_orig) free(fs_orig);
 	return -1;
 }
