@@ -2,7 +2,7 @@
 #  customize.tcl
 
 # -----------------------------------------------------------
-#  Copyright (C) 2002-2003 Tresys Technology, LLC
+#  Copyright (C) 2002-2005 Tresys Technology, LLC
 #  see file 'COPYING' for use and warranty information 
 #
 #  Requires tcl and tk 8.3+, with BWidgets
@@ -1090,20 +1090,6 @@ proc Sepct_Customize::createNoteBook { tabsBox } {
 	return 0
 }
 
-####################################################################
-# ::change_tab_state
-#
-#	Called to enable or disable customize tab
-#
-proc Sepct_Customize::change_tab_state { nb cmd } {
-	if {$cmd == "disable" } {
-		$nb itemconfigure  $Sepct::customize_tab -state disabled
-	} else {
-		$nb itemconfigure  $Sepct::customize_tab -state normal
-	}
-	return
-}
-
 ###################################################################
 # ::create
 #  	- Creates all major widgets and frames.
@@ -1947,7 +1933,7 @@ proc Sepct_Customize::create_unUsed_Modules_Dir {} {
 	# First, check to see if the unUsed directory exists. 
 	if { [file exists $Sepct_Customize::used_Modules_Dir] } {
 		set rt [catch {file mkdir $Sepct_Customize::unUsed_Modules_Dir} err]
-		if {rt != 0} {
+		if {$rt != 0} {
 			return -code error $err
 		}
 		set rt [Sepct_db::addTree_node $Sepct_Customize::unUsed_Modules_Dir 0 0]
